#!/usr/bin/perl
#   ============================================================================
#   @file   changepath.pl
#
#   @path   $(DSPLINK)/etc/host/tools/PrOS/Davinci/
#
#   @desc   Change the path to executable and test files to comply with
#           path format supported by PrFile.
#
#   @ver    1.65.00.03
#   ============================================================================
#   Copyright (C) 2002-2009, Texas Instruments Incorporated -
#   http://www.ti.com/
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#   
#   *  Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#   
#   *  Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#   
#   *  Neither the name of Texas Instruments Incorporated nor the names of
#      its contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#   
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#   ============================================================================


#   ----------------------------------------------------------------------------
#   Run this script from the directory that contains test directory.
#   ----------------------------------------------------------------------------

use strict ;

if ($#ARGV < 1)
{
    print "Usage: [PERL] changepath.pl <directory listing> <hashtable>\n" ;
    exit ;
}

my ($listing) = $ARGV[0] ;
my ($hashtab) = $ARGV[1] ;

open (FH, "<$listing") || die "!! Couldn't open $listing\n" ;
my (@listOfFiles) = <FH> ;
close (FH) ;

open (FH, "<$hashtab") || die "!! Couldn't open $hashtab\n" ;
my (@hashNames) = <FH> ;
close (FH) ;

# Populate the hashtable
my (%hash) ;
my (@allkeys) ;

foreach my $line (@hashNames)
{
    if (!($line =~ m/^#/))
    {
        chomp ($line) ;
        my ($old, $new) = split (/\s+\|\s+/, $line) ;
        my (@path) = split (/\\/, $old) ;
        $hash {$path[$#path]} = $new ;
    }
}

@allkeys = keys %hash ;

# Create the directories
mkdir ("./TESTDATA") ;
mkdir ("./TESTDATA/opt") ;
mkdir ("./TESTDATA/opt/dsplink") ;
mkdir ("./TESTDATA/opt/dsplink/test") ;
mkdir ("./TESTDATA/opt/dsplink/test/api") ;
mkdir ("./TESTDATA/opt/dsplink/test/analysis") ;
mkdir ("./TESTDATA/opt/dsplink/test/stress") ;
mkdir ("./TESTDATA/opt/dsplink/test/behavior") ;
mkdir ("./TESTDATA/opt/dsplink/test/dsp") ;
mkdir ("./TESTDATA/opt/dsplink/test/dsp/bin") ;

foreach my $line (@listOfFiles)
{
    if ($line =~ m/^F/)
    {
        my ($type, $name) = split (/\s+/, $line) ;

        my (@path) = split (/\\/, $name) ;
        if (   ($name =~ m/PROC/)
            || ($name =~ m/MSGQ/)
            || ($name =~ m/CHNL/))
        {

            if ($hash {$path[$#path]})
            {
                open (FH, "<$name") || die "!! Couldn't open $name\n" ;
                my (@alllines) = <FH> ;
                close (FH) ;

                if ($name =~ m/Davinci/)
                {
                    # Filter out the external memory scenarios from existing test files
                    my ($newFileName) = "" ;
                    $newFileName = "./TESTDATA/" . $hash {$path[$#path]};
                    print "CREATING FILE: $newFileName\n" ;

                    unlink ($name) ;
                    open (FH, ">$name") || die "!! Couldn't open $name\n" ;
                    open (NFH, ">$newFileName") || die "!! Couldn't open $newFileName\n" ;

                    foreach my $l (@alllines)
                    {
                        if ($l =~ m/^!/) # Spit out all comment lines
                        {
                            print FH $l ;
                            print NFH $l ;
                        }
                        else
                        {
                            if (!(($l =~ m/_bios/) || ($l =~ m/_text/)))
                            {
                                print FH $l ;
                            }

                            if (!(($l =~ m/_bios/) || ($l =~ m/_text/)))
                            {
                                # If the line contains a 'old' file name, then replace
                                # it with new name
                                chomp ($l) ;
                                my ($key) = &FindKey ($l) ;
                                if ($key ne "")
                                {
                                    my (@fields) = split (/ /, $l) ;
                                    foreach my $f (@fields)
                                    {
                                        if ($f =~ m/$key/)
                                        {
                                            $f =~ s/\//\\/g ;
                                            print NFH "a:" . $f . " ";
                                        }
                                        else
                                        {
                                            print NFH "$f " ;
                                        }
                                    }
                                    print NFH "\n" ;

                                }
                                else
                                {
                                    print NFH "$l\n" ;
                                }
                            }
                        }
                    }
                    close (FH);
                    close (NFH) ;
                }
            }
        }
    }
}

sub FindKey ()
{
    my ($text) = @_ ;

    foreach my $key (@allkeys)
    {
        if ($text =~ m#$key#)
        {
            return $key ;
        }
    }

    return "" ;
}
