/** ============================================================================
 *  @file   omap3530_dspclk.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/OMAP3530/shmem/WinCE/
 *
 *  @desc   Implementation of dsp clk functionality.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <omap3530_hal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_DSPCLK

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON TRC_SetReason (status,FID_C_ARCH_DSPCLK,__LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/*  ============================================================================
 *  @const  CM_CLKSEL1_PLL_IVA2_OFFSET, CM_CLKSEL2_PLL_IVA2_OFFSET
 *
 *  @desc   Offsets from the IVA2_CM base address
 *  ============================================================================
 */
#define CM_CLKSEL1_PLL_IVA2_OFFSET        0x40
#define CM_CLKSEL2_PLL_IVA2_OFFSET        0x44

/*  ============================================================================
 *  @const  DPLL_MULT_MASK, DPLL_MULT_SHIFT, DPLL_DIV_MASK, DPLL_DIV_SHIFT,DPLL_DIV_SHIFT, 
 *          DPLL_IVA2_CLKOUT_DIV_SHIFT
 *
 *  @desc   Masks and SHIFTS for DPLL multiplier and divisor
 *  ============================================================================
 */
#define DPLL_MULT_MASK                    0x0007FF00
#define DPLL_MULT_SHIFT                   0x8
#define DPLL_DIV_MASK                     0x0000003F
#define DPLL_DIV_SHIFT                    0x0
#define DPLL_IVA2_CLKOUT_DIV_SHIFT        0x0

/*  ============================================================================
 *  @const  PRM_CLKSEL_OFFSET, PRM_CLKSEL_MASK
 *
 *  @desc   Offsets from the IVA2_PRM base address
 *  ============================================================================
 */
#define PRM_CLKSEL_OFFSET                 0xD40
#define PRM_CLKSEL_MASK                   0x7


/** ============================================================================
 *  @func   OMAP3530_getDspClkRate
 *
 *  @desc   Get DSP clock frequency
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
OMAP3530_getDspClkRate (IN Pvoid dspIdentifier, OUT Uint32 * cpuFreq)
{
    DSP_STATUS        status    = DSP_SOK ;
    OMAP3530_HalObj * halObject = NULL  ;
    Uint32            dpll_val  = 0;    
    Uint32            sys_clk_idx =0;
    Uint32            sys_clk_values[]={12000,13000,19200,26000,38400,16800,0,0}; // in khz
    

    TRC_2ENTER ("OMAP3530_getDspClkRate", dspIdentifier, cpuFreq) ;

    DBC_Require (dspIdentifier != NULL) ;
    DBC_Require (cpuFreq != NULL) ;
    halObject = (OMAP3530_HalObj *) dspIdentifier ;

    sys_clk_idx=(REG(halObject->iva2PrmBase + PRM_CLKSEL_OFFSET)& PRM_CLKSEL_MASK);       
    dpll_val = REG (halObject->iva2CmBase + CM_CLKSEL1_PLL_IVA2_OFFSET);
    *cpuFreq = (((dpll_val & DPLL_MULT_MASK)>>DPLL_MULT_SHIFT) * sys_clk_values[sys_clk_idx])/
                (((dpll_val & DPLL_DIV_MASK) >> DPLL_DIV_SHIFT) + 1);
    dpll_val = REG (halObject->iva2CmBase + CM_CLKSEL2_PLL_IVA2_OFFSET);    
    *cpuFreq = *cpuFreq/(dpll_val >> DPLL_IVA2_CLKOUT_DIV_SHIFT);
    
    PRINT_Printf("DSP Clk Rate %d\n", *cpuFreq) ;
    
    TRC_1LEAVE ("OMAP3530_getDspClkRate", status) ;

    return status ;
}




#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
