/** ============================================================================
 *  @file   tnetv107xgem_phy_shmem.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/TNETV107XGEM/shmem/WinCE/
 *
 *  @desc   WinCE shmem driver interface file.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */

 
 /*  ----------------------------------- OS Headers                  */
#include <windows.h>
#include <ceddk.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <tnetv107xgem_hal.h>
#include <tnetv107xgem_hal_boot.h>
#include <tnetv107xgem_hal_intgen.h>
#include <tnetv107xgem_hal_pwr.h>
#include <tnetv107xgem_phy_shmem.h>


#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_SHMEM_PHY

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON TRC_SetReason (status,FID_C_ARCH_PHY_SHMEM,__LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @name   TNETV107XGEM_shmemInterface
 *
 *  @desc   Interface functions exported by the Shared Driver subcomponent.
 *  ============================================================================
 */
HAL_Interface TNETV107XGEM_shmemInterface = {&TNETV107XGEM_phyShmemInit,
                                          &TNETV107XGEM_phyShmemExit,
                                          &TNETV107XGEM_halBootCtrl,
                                          &TNETV107XGEM_halIntCtrl,
                                          NULL,
                                          NULL,
                                          &TNETV107XGEM_halPwrCtrl,
                                          NULL,
                                          NULL,
                                          NULL,
                                          NULL} ;


/* ============================================================================
 *  @func   TNETV107XGEM_phyShmemInit
 *
 *  @desc   Initializes Shared/Davinci Gem device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
TNETV107XGEM_phyShmemInit (IN Pvoid halObj)
{
    DSP_STATUS        status    = DSP_SOK ;
    TNETV107XGEM_HalObj * halObject = NULL    ;
    MemMapInfo  mapInfo ;
        

    TRC_1ENTER ("TNETV107XGEM_phySharedInit", halObj) ;

    DBC_Require (NULL != halObj) ;

    halObject = (TNETV107XGEM_HalObj *) halObj ;

    /* map INTD regs */
    mapInfo.src  = INTD_BASE  - (INTD_BASE % PAGE_SIZE) ;
    mapInfo.size = INTD_MAP_SIZE; // assuming page_size multiple 
    mapInfo.memAttrs = MEM_UNCACHED ;
    status = MEM_Map (&mapInfo) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
        halObject->intdBase = 0 ;
    }
    else {
        halObject->intdBase = mapInfo.dst + (INTD_BASE % PAGE_SIZE);
    }

    /* map CHIP CFG regs */
    mapInfo.src  = CHIP_CFG_BASE  - (CHIP_CFG_BASE % PAGE_SIZE) ;
    mapInfo.size = CHIP_CFG_MAP_SIZE; // assuming page_size multiple 
    mapInfo.memAttrs = MEM_UNCACHED ;
    status = MEM_Map (&mapInfo) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
        halObject->cfgBase = 0 ;
    }
    else {
        halObject->cfgBase = mapInfo.dst + (CHIP_CFG_BASE % PAGE_SIZE);
    }

    /* map CLk PSC regs */
    mapInfo.src  = CLK_CTRL_BASE - (CLK_CTRL_BASE % PAGE_SIZE) ;
    mapInfo.size = CLK_CTRL_MAP_SIZE; // assuming page_size multiple 
    mapInfo.memAttrs = MEM_UNCACHED ;
    status = MEM_Map (&mapInfo) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
        halObject->clkCtrlBase = 0 ;
    }
    else {
        halObject->clkCtrlBase = mapInfo.dst + (CLK_CTRL_BASE % PAGE_SIZE);
    }
    
    TRC_1LEAVE ("TNETV107XGEM_phySharedInit", status) ;

    return status ;
}


/* ============================================================================
 *  @func   TNETV107XGEM_phyShmemExit
 *
 *  @desc   Finalizes Shared/Davinci Gem device.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
TNETV107XGEM_phyShmemExit (IN Pvoid halObj)
{
    DSP_STATUS     status = DSP_SOK ;    
    MemUnmapInfo      unmapInfo                              ;
    TNETV107XGEM_HalObj * halObject = NULL    ;
    
    TRC_1ENTER ("TNETV107XGEM_phySharedExit", halObj) ;

    DBC_Require (NULL != halObj) ;
    
    halObject = (TNETV107XGEM_HalObj *) halObj ;
    
    unmapInfo.addr = halObject->intdBase - (halObject->intdBase % PAGE_SIZE);
    unmapInfo.size = INTD_MAP_SIZE;
    if (unmapInfo.addr != 0) {
        MEM_Unmap (&unmapInfo) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
        halObject->intdBase = 0 ;
    }

    unmapInfo.addr = halObject->cfgBase - (halObject->cfgBase % PAGE_SIZE);
    unmapInfo.size = CHIP_CFG_MAP_SIZE;
    if (unmapInfo.addr != 0) {
        MEM_Unmap (&unmapInfo) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
        halObject->cfgBase = 0 ;
    }

    unmapInfo.addr = halObject->clkCtrlBase - (halObject->clkCtrlBase % PAGE_SIZE);
    unmapInfo.size = CLK_CTRL_MAP_SIZE;
    if (unmapInfo.addr != 0) {
        MEM_Unmap (&unmapInfo) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
        halObject->clkCtrlBase = 0 ;
    }

    (void) halObj ;

    TRC_1LEAVE ("TNETV107XGEM_phySharedExit", status) ;

    return status ;
}


#if defined (__cplusplus)

}
#endif
