/** ============================================================================
 *  @file   drv_pmgr.c
 *
 *  @path   $(DSPLINK)/gpp/src/pmgr/WinCE/
 *
 *  @desc   Implementation of Wince module driver interface.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  Defined to include MACROS EXPORT_SYMBOL. This must be done before including
 *  module.h
 */
#if !defined (EXPORT_SYMTAB)
#define EXPORT_SYMTAB
#endif

/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>
#include <ceddk.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <gpptypes.h>
#include <dsplink.h>
#include <_dsplink.h>
#include <procdefs.h>
#include <version.h>

#if defined (RINGIO_COMPONENT)
#include <ringiodefs.h>
#endif /* if defined (RINGIO_COMPONENT) */
#if defined (MPLIST_COMPONENT)
#include <mplistdefs.h>
#endif /* if defined (MPLIST_COMPONENT) */
#if defined (MSGQ_COMPONENT)
#include <msgqdefs.h>
#endif /* if defined (MSGQ_COMPONENT) */
#if defined (CHNL_COMPONENT)
#include <chnldefs.h>
#endif /* if defined (CHNL_COMPONENT) */

/*  ----------------------------------- Profiling                   */
#if defined (DDSP_PROFILE)
#include <profile.h>
#endif /* if defined (DDSP_PROFILE) */

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- Generic Functions           */

#include <list.h>
#include <idm.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>
#include <mem.h>
#include <mem_os.h>
#include <prcs.h>
#if defined (NOTIFY_COMPONENT)
#include <notify_knl.h>
#endif /* #if defined (NOTIFY_COMPONENT) */

/*  ----------------------------------- User API                    */
#if defined (MPCS_COMPONENT)
#include <mpcs.h>
#endif /* if defined (MPCS_COMPONENT) */

/*  ----------------------------------- Processor Manager           */
#include <drvdefs.h>
#include <pmgr_proc.h>
#if defined (CHNL_COMPONENT)
#include <pmgr_chnl.h>
#endif /* if defined (CHNL_COMPONENT) */
#if defined (MSGQ_COMPONENT)
#include <pmgr_msgq.h>
#endif /* if defined (MSGQ_COMPONENT) */

/*  ----------------------------------- Link Driver                 */
#if defined (CHNL_COMPONENT)
#include <ldrv_data.h>
#include <ldrv_chnl.h>
#endif /* if defined (CHNL_COMPONENT) */
#if defined (RINGIO_COMPONENT)
#include <ldrv_ringio.h>
#endif /* if defined (RINGIO_COMPONENT) */
#if defined (MPLIST_COMPONENT)
#include <ldrv_mplist.h>
#endif /* if defined (MPLIST_COMPONENT) */
#if defined (POOL_COMPONENT)
#include <ldrv_pool.h>
#endif /* if defined (POOL_COMPONENT) */
#if defined (MPCS_COMPONENT)
#include <ldrv_mpcs.h>
#endif /* if defined (MPCS_COMPONENT) */
#include <ldrv.h>
#include <ldrv_proc.h>
#if defined (MSGQ_COMPONENT)
#include <ldrv_mqt.h>
#include <ldrv_msgq.h>
#endif /* if defined (MSGQ_COMPONENT) */
#if !(defined (ONLY_PROC_COMPONENT))
#include <ips.h>
#endif

/*  ----------------------------------- LOG  Headers                 */
#if defined (LOG_COMPONENT)
#include <linklog.h>
#endif /* if defined (LOG_COMPONENT) */


EXPORT_SYMBOL(LDRV_PROC_getState);
EXPORT_SYMBOL(LDRV_PROC_setState);
EXPORT_SYMBOL(PMGR_PROC_getSymbolAddress);
#if !(defined (ONLY_PROC_COMPONENT))
EXPORT_SYMBOL(IPS_register);
EXPORT_SYMBOL(IPS_notify);
EXPORT_SYMBOL(IPS_unregister);
#endif



#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_OSAL_DRV

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON TRC_SetReason (status, FID_C_OSAL_DRV_PMGR, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */

/** ============================================================================
 *  @macro  SIGNATURE
 *
 *  @desc   Driver Object Signature
 *  ============================================================================
 */
#define SIGNATURE   0x5f52474d      /* Letters "DRV_" (in reverse) */

/** ============================================================================
 *  @macro  MAX_ARGUMENT_VAL_LENGTH
 *
 *  @desc   Denotes the Maximum number of bytes supported for each Argument.
 *  ============================================================================
 */
#define MAX_ARGUMENT_VAL_LENGTH   30      /* Max Byte Count for an Argument */

/** ============================================================================
 *  @name   DRV_OBJECT
 *
 *  @desc   Windows CE specific PROC MGR Device Driver Object.
 *
 *  @field  signature
 *              Signature of the DPC object.
 *  @field  traceMask
 *              Index of the DPC object.
 *  @field  initialized
 *
 *  @field  refCount
 *
 *  @see    DPC_DpcThreadInfo
 *  ============================================================================
 */
typedef struct DRV_OBJECT {
    Uint32       signature;
    Uint32       traceMask;
    Bool         initialized;
    Uint32       refCount;
} DRV_OBJECT;

/** ----------------------------------------------------------------------------
 *  @name   DRV_IsInitialized
 *
 *  @desc   If the module has been initialized or not.
 *  ----------------------------------------------------------------------------
 */
STATIC Bool DRV_IsInitialized = FALSE ;

/** ============================================================================
 *  @name   LDRV_LinkCfgPtr
 *
 *  @desc   Global pointer containing DSPLINK configuration information for the
 *          driver.
 *  ============================================================================
 */
extern LINKCFG_Object * LDRV_LinkCfgPtr ;


#if defined (MSGQ_COMPONENT)
/** ----------------------------------------------------------------------------
 *  @name   DRV_MaxMsgqs
 *
 *  @desc   Maximum number of message queues on the GPP.
 *  ----------------------------------------------------------------------------
 */
STATIC Uint32 DRV_MaxMsgqs = 0 ;
#endif /* if defined (CHNL_COMPONENT) */
/** ----------------------------------------------------------------------------
 *  @name   hDriverObject
 *
 *  @desc   PROC MGR Driver Object Instance.
 *  ----------------------------------------------------------------------------
 */
STATIC DRV_OBJECT  hDrvObj;    

#if defined (CHNL_COMPONENT)
/** ----------------------------------------------------------------------------
 *  @name   DRV_MemAllocAttrs
 *
 *  @desc   Allocation arguments for CHNL ID to POOL ID Map table allocation.
 *          These are filled up by alloc function and used by free function.
 *  ----------------------------------------------------------------------------
 */
STATIC MemAllocAttrs   DRV_MemAllocAttrs ;

/** ----------------------------------------------------------------------------
 *  @name   DRV_ChnlIdToPoolId
 *
 *  @desc   Place holder to store pool ids for all channels.
 *  ----------------------------------------------------------------------------
 */
STATIC Uint32 (*DRV_ChnlIdToPoolId) [MAX_CHANNELS] ;
#endif /* if defined (CHNL_COMPONENT) */


/** ----------------------------------------------------------------------------
 *  @name   DRV_CallAPI
 *
 *  @desc   Function to invoke the APIs through ioctl.
 *
 *  @arg    cmd
 *              Command identifier.
 *          args
 *              Arguments to be passed for the command.
 *
 *  @ret    DSP_SOK
 *              Operation successfully completed.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
STATIC
NORMAL_API
DSP_STATUS
DRV_CallAPI (Uint32 cmd, CMD_Args * args) ;


/** ----------------------------------------------------------------------------
 *  @func  DSPLINK_Cleanup
 *
 *  @desc  Kernel side component of the cleanup utility.
 *
 *  @arg    None
 *
 *  @ret    None
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
STATIC
NORMAL_API
Void
DSPLINK_Cleanup (Void) ;



/** ----------------------------------------------------------------------------
 *  @func   DRV_InitializeModule
 *
 *  @desc   Module initialization function for Wince driver.
 *
 *  @arg    None.
 *
 *  @ret    0
 *              Successful execution.
 *          non-zero
 *              Failure.
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ----------------------------------------------------------------------------
 */
STATIC int DRV_InitializeModule (LPCTSTR pContext);


/** ----------------------------------------------------------------------------
 *  @func   DRV_FinalizeModule
 *
 *  @desc   Module finalization  function for Wince driver.
 *
 *  @arg    None.
 *
 *  @ret    None.
 *
 *  @enter  None
 *
 *  @leave  None
 *
 *  @see    None
 *  ----------------------------------------------------------------------------
 */
STATIC void  DRV_FinalizeModule (void) ;



/** ----------------------------------------------------------------------------
 *  @func   DRV_InitializeModule
 *
 *  @desc   Module initialization  function for Wince driver.
 *  ----------------------------------------------------------------------------
 */
STATIC
int DRV_InitializeModule (LPCTSTR pContext)
{
    int             result = 0       ;
    DSP_STATUS      status = DSP_SOK ;
#if defined (CHNL_COMPONENT)
    Uint32          tabSize          ;
#endif /* if defined (CHNL_COMPONENT) */

    TRC_0ENTER ("DRV_InitializeModule") ;
    /* Display the version info and created date/time */
    PRINT_Printf ("DSPLINK Module (%s) created on Date: %s Time: %s\n",
                  DSPLINK_VERSION,
                  __DATE__,
                  __TIME__) ;
    status = OSAL_Initialize () ;

#if defined (NOTIFY_COMPONENT)
    /* Initialize the User Event module. */
    UEVENT_Initialize () ;
#endif /* #if defined (NOTIFY_COMPONENT) */




    if (DSP_SUCCEEDED (status)) {
        /*  --------------------------------------------------------------------
         *  To enable trace for a component and/or subcomponent, uncomment the
         *  corresponding statements below. (This is not a comprehensive list
         *  of available trace masks. See file signature.h)
         *  --------------------------------------------------------------------
         */

        //TRC_ENABLE (ID_PMGR_ALL)        ; 
        //TRC_ENABLE (ID_GEN_ALL)         ;
        //TRC_ENABLE (ID_OSAL_ALL)        ;
        //TRC_ENABLE (ID_LDRV_ALL)        ;
        //TRC_ENABLE (ID_LDRV_DATA_ALL)   ;
        //TRC_ENABLE (ID_LDRV_DRV_ALL)    ;
        //TRC_ENABLE (ID_LDRV_IPS_ALL)    ;
        //TRC_ENABLE (ID_LDRV_MQT_ALL)    ;
        //TRC_ENABLE (ID_LDRV_POOL_ALL)   ;
        //TRC_ENABLE (ID_ARCH_ALL)   ;
        //TRC_ENABLE (ID_ARCH_HAL_ALL)   ;
        //TRC_ENABLE (ID_ARCH_DSP_ALL)   ;

        /* TRC_ENABLE (ID_PMGR_PROC)       ; */
        /* TRC_ENABLE (ID_PMGR_CHNL)       ; */
        /* TRC_ENABLE (ID_PMGR_MSGQ)       ; */

        /* TRC_ENABLE (ID_GEN_UTILS)       ; */
        /* TRC_ENABLE (ID_GEN_LIST)        ; */
        /* TRC_ENABLE (ID_GEN_COFF)        ; */
        /* TRC_ENABLE (ID_GEN_COFF_55x)    ; */
        /* TRC_ENABLE (ID_GEN_COFF_64x)    ; */
        /* TRC_ENABLE (ID_GEN_BINLOADER)   ; */
        /* TRC_ENABLE (ID_GEN_IDM )        ; */
        /* TRC_ENABLE (ID_GEN_NOLOADER)    ; */
        /* TRC_ENABLE (ID_GEN_COFF_MEM)    ; */
        /* TRC_ENABLE (ID_GEN_COFF_FILE)   ; */
        /* TRC_ENABLE (ID_GEN_COFF_SHM)    ; */

        /* TRC_ENABLE (ID_OSAL)            ; */
        /* TRC_ENABLE (ID_OSAL_DPC)        ; */
        /* TRC_ENABLE (ID_OSAL_ISR)        ; */
        /* TRC_ENABLE (ID_OSAL_KFILE)      ; */
        /* TRC_ENABLE (ID_OSAL_MEM)        ; */
        /* TRC_ENABLE (ID_OSAL_PRCS)       ; */
        /* TRC_ENABLE (ID_OSAL_SYNC)       ; */
        /* TRC_ENABLE (ID_OSAL_DRV)        ; */
        //TRC_ENABLE (ID_OSAL_NOTIFY_KNL) ;
        /* TRC_ENABLE (ID_OSAL_USER)       ; */
        /* TRC_ENABLE (ID_OSAL_KFILE_PSEUDO) ; */

        /* TRC_ENABLE (ID_LDRV)            ; */
        /* TRC_ENABLE (ID_LDRV_PROC)       ; */
        /* TRC_ENABLE (ID_LDRV_CHNL)       ; */
        /* TRC_ENABLE (ID_LDRV_MSGQ)       ; */
        /* TRC_ENABLE (ID_LDRV_CHIRPS)     ; */
        /* TRC_ENABLE (ID_LDRV_MPCS)       ; */
        /* TRC_ENABLE (ID_LDRV_MPLIST)     ; */
        /* TRC_ENABLE (ID_LDRV_RINGIO)     ; */
        /* TRC_ENABLE (ID_LDRV_SMM)        ; */
        /* TRC_ENABLE (ID_LDRV_OS)         ; */
        /* TRC_ENABLE (ID_LDRV_DATA)       ; */
        /* TRC_ENABLE (ID_LDRV_DATA_PCPY)  ; */
        /* TRC_ENABLE (ID_LDRV_DATA_ZCPY)  ; */
        /* TRC_ENABLE (ID_LDRV_DRV)        ; */
        /* TRC_ENABLE (ID_LDRV_DRV_SHM)    ; */
        //TRC_ENABLE (ID_LDRV_IPS)        ; 
        //TRC_ENABLE (ID_LDRV_IPS_IPS)    ; 
        /* TRC_ENABLE (ID_LDRV_MQT)        ; */
        /* TRC_ENABLE (ID_LDRV_MQT_PCPY)   ; */
        /* TRC_ENABLE (ID_LDRV_MQT_ZCPY)   ; */
        /* TRC_ENABLE (ID_LDRV_POOL)       ; */
        /* TRC_ENABLE (ID_LDRV_POOL_BUF)   ; */
        /* TRC_ENABLE (ID_LDRV_POOL_DMA)   ; */
        /* TRC_ENABLE (ID_LDRV_POOL_SMA)   ; */

        /* TRC_ENABLE (ID_ARCH_DSP)        ; */
        /* TRC_ENABLE (ID_ARCH_DSP_INTF)   ; */
        /* TRC_ENABLE (ID_ARCH_CFG)        ; */
        /* TRC_ENABLE (ID_ARCH_HAL)        ; */
        /* TRC_ENABLE (ID_ARCH_HAL_MAP)    ; */
        /* TRC_ENABLE (ID_ARCH_HAL_INTGEN) ; */
        /* TRC_ENABLE (ID_ARCH_HAL_DMA)    ; */
        /* TRC_ENABLE (ID_ARCH_HAL_RDWR)   ; */
        /* TRC_ENABLE (ID_ARCH_HAL_BOOT)   ; */
        /* TRC_ENABLE (ID_ARCH_HAL_PWR)    ; */
        /* TRC_ENABLE (ID_ARCH_HAL_MMU)    ; */
        /* TRC_ENABLE (ID_ARCH_DSPCLK)     ; */

        /* TRC_ENABLE (ID_ARCH_SHMEM_PHY)  ; */
        /* TRC_ENABLE (ID_ARCH_PCI_PHY)    ; */
        /* TRC_ENABLE (ID_ARCH_VLYNQ_PHY)  ; */


        /*  --------------------------------------------------------------------
         *  To set desired severity level for trace, uncomment the statement
         *  below and change the argument to the function below.
         *  --------------------------------------------------------------------
         */

        //TRC_SET_SEVERITY (TRC_ENTER) ;

        
#if defined (CHNL_COMPONENT)
            tabSize = sizeof (Uint32) * MAX_DSPS * MAX_CHANNELS ;
            status = MEM_Calloc ((Void **) &DRV_ChnlIdToPoolId,
                                 tabSize,
                                 &DRV_MemAllocAttrs) ;

            if (DSP_FAILED (status)) {
                result = -1 ;
                SET_FAILURE_REASON ;
            }
        
#endif /* if defined (CHNL_COMPONENT) */

        if (DSP_SUCCEEDED (status)) {
            DRV_IsInitialized = TRUE ;
        }
    }
    else {
        SET_FAILURE_REASON ;
        result = -1 ;
    }

    TRC_1LEAVE ("DRV_InitializeModule", result) ;

    return result ;
}


/** ----------------------------------------------------------------------------
 *  @name   DRV_FinalizeModule
 *
 *  @desc   Wince driver function to finalize the driver module.
 *  ----------------------------------------------------------------------------
 */
STATIC
NORMAL_API
void DRV_FinalizeModule (void)
{
    DSP_STATUS   status    = DSP_SOK ;

#if defined (CHNL_COMPONENT)
    MemFreeAttrs freeAttrs ;
#endif /* if defined (CHNL_COMPONENT) */

    TRC_0ENTER ("DRV_FinalizeModule") ;

#if defined (CHNL_COMPONENT)
    freeAttrs.physicalAddress = DRV_MemAllocAttrs.physicalAddress ;
    freeAttrs.size = sizeof (Uint32) * MAX_DSPS * MAX_CHANNELS ;

    status = MEM_Free ((Pvoid *) &DRV_ChnlIdToPoolId, &freeAttrs) ;

    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }
#endif /* if defined (CHNL_COMPONENT) */

    
    DRV_IsInitialized = FALSE ;

    TRC_0LEAVE ("DRV_FinalizeModule") ;

#if defined (NOTIFY_COMPONENT)
    UEVENT_Finalize () ;
#endif /* #if defined (NOTIFY_COMPONENT) */

    status = OSAL_Finalize () ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

}





/** ----------------------------------------------------------------------------
 *  @name   DRV_CallAPI
 *
 *  @desc   Function to invoke the APIs through ioctl.
 *
 *  @modif  None.
 *  ----------------------------------------------------------------------------
 */
STATIC
NORMAL_API
DSP_STATUS
DRV_CallAPI (Uint32 cmd, CMD_Args * args)
{
    DSP_STATUS status    = DSP_SOK ; /* status of driver's ioctl    */
    DSP_STATUS retStatus = DSP_SOK ; /* status of the PMGR function */
    HRESULT hr           = S_OK    ; /* Used for mapping result code */
    LPVOID imgMapPtr     = NULL    ; /* Used to Map Strings into Driver   */
    LPVOID argMapPtr     = NULL    ; /* Used to Map Arguments into Driver */
    ProcAttr * attr      = NULL    ; /* Used for PROC_ATTACH Arguments    */
    Uint16  argCount     = 0       ; /* Used to Track No. of Arguments */
#if defined (CHNL_COMPONENT)
    Uint32 tabSize                 ;
#endif /* if defined (CHNL_COMPONENT) */

    TRC_2ENTER ("DRV_CallAPI", cmd, args) ;

    args->apiStatus = DSP_SOK ;

    switch (cmd) {

#if defined (MSGQ_COMPONENT)
    case CMD_MSGQ_PUT:
        {
#if defined (LOG_GD_MSGQ_PUT)
            /* Log the event */
            DSPLINKLOG_LogEvent (GD_MSGQ_PUT,
                                 args->apiArgs.msgqPutArgs.msg->msgId,
                                 (Uint32) args->apiArgs.msgqPutArgs.msgqQueue,
                                 (Uint32) args->apiArgs.msgqPutArgs.msg,
                                 (Uint32) (args->apiArgs.msgqPutArgs.msg)->size,
                                 0,
                                 0) ;
#endif /* if defined (LOG_GD_MSGQ_PUT) */

            // Validate msg size > 0 (0 size is a common issue if MSG memory is memset to 0 after alloc)
            if ((args->apiArgs.msgqPutArgs.msg)->size == 0)
            {
                retStatus = DSP_EINVALIDARG;
                SET_FAILURE_REASON ;
            }
            else
            {
                
                // Msg is allocated from the pool and usr/kernel address translation done at API level.
                // No pointer mapping required here.
                retStatus = PMGR_MSGQ_put (args->apiArgs.msgqPutArgs.msgqQueue,
                                            args->apiArgs.msgqPutArgs.msg) ;
            }
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_MSGQ_GET:
        {
            MsgqMsg msg ;
            retStatus = PMGR_MSGQ_get (args->apiArgs.msgqGetArgs.msgqQueue,
                                       args->apiArgs.msgqGetArgs.timeout,
                                       &msg) ;
            if (DSP_SUCCEEDED (retStatus)) {
                args->apiArgs.msgqGetArgs.poolId  = msg->poolId ;
                args->apiArgs.msgqGetArgs.msgAddr = (Uint32) msg ;
            }
            args->apiStatus = retStatus ;
        }
        break ;
#endif /* if defined (MSGQ_COMPONENT) */

#if defined (CHNL_COMPONENT)
    case CMD_CHNL_ISSUE:
        {
            ProcessorId     procId  = args->apiArgs.chnlIssueArgs.procId ;
            ChannelId       chnlId  = args->apiArgs.chnlIssueArgs.chnlId ;
            ChannelIOInfo * ioInfo  = NULL;
            hr = CeOpenCallerBuffer((PVOID*) &ioInfo, 
                args->apiArgs.chnlIssueArgs.ioReq,
                sizeof (ChannelIOInfo), ARG_IO_PTR, FALSE);

#if defined (LOG_GD_CHNL_I_START)
            /* Log the event */
            DSPLINKLOG_LogEvent (
                        GD_CHNL_I_START,
                        0,
                        (((procId << 16) & 0xFFFF0000) | (chnlId & 0xFFFF)),
                        (Uint32) ioInfo->buffer,
                        ioInfo->size,
                        0,
                        0) ;
#endif /* if defined (LOG_GD_CHNL_I_START) */

            retStatus = PMGR_CHNL_issue (procId, chnlId, ioInfo, NULL) ;

#if defined (LOG_GD_CHNL_I_COMPLETE)
            /* Log the event */
            DSPLINKLOG_LogEvent (
                          GD_CHNL_I_COMPLETE,
                          0,
                          (((procId << 16) & 0xFFFF0000) | (chnlId & 0xFFFF)),
                          (Uint32) ioInfo->buffer,
                          ioInfo->size,
                          0,
                          0) ;
#endif /* if defined (LOG_GD_CHNL_I_COMPLETE) */

            args->apiStatus = retStatus ;
            hr = CeCloseCallerBuffer((PVOID) ioInfo, 
                args->apiArgs.chnlIssueArgs.ioReq, 
                sizeof (ChannelIOInfo), ARG_IO_PTR);
        }
        break ;

    case CMD_CHNL_RECLAIM:
        {
            ProcessorId     procId  = args->apiArgs.chnlReclaimArgs.procId  ;
            ChannelId       chnlId  = args->apiArgs.chnlReclaimArgs.chnlId  ;
            Uint32          timeout = args->apiArgs.chnlReclaimArgs.timeout ;
            ChannelIOInfo * ioInfo  = NULL;
            hr = CeOpenCallerBuffer((PVOID*) &ioInfo, 
                args->apiArgs.chnlReclaimArgs.ioReq,
                sizeof (ChannelIOInfo), ARG_IO_PTR, FALSE);
            DEBUGCHK(SUCCEEDED(hr));

#if defined (LOG_GD_CHNL_R_START)
            /* Log the event */
            DSPLINKLOG_LogEvent (
                          GD_CHNL_R_START,
                          0,
                          (((procId << 16) & 0xFFFF0000) | (chnlId & 0xFFFF)),
                          (Uint32) NULL,
                          ioInfo->size,
                          0,
                          0) ;
#endif /* if defined (LOG_GD_CHNL_R_START) */

            retStatus = PMGR_CHNL_reclaim (procId, chnlId, timeout, ioInfo, NULL) ;

#if defined (LOG_GD_CHNL_R_COMPLETE)
            /* Log the event */
            DSPLINKLOG_LogEvent (
                          GD_CHNL_R_COMPLETE,
                          0,
                          (((procId << 16) & 0xFFFF0000) | (chnlId & 0xFFFF)),
                          (Uint32) ioInfo->buffer,
                          ioInfo->size,
                          0,
                          0) ;
#endif /* if defined (LOG_GD_CHNL_R_COMPLETE) */

            args->apiStatus = retStatus ;
            hr = CeCloseCallerBuffer((PVOID) ioInfo, 
                args->apiArgs.chnlReclaimArgs.ioReq, 
                sizeof (ChannelIOInfo), ARG_IO_PTR);
        }
        break ;
#endif /* if defined (CHNL_COMPONENT) */

#if defined (MSGQ_COMPONENT)
    case CMD_MSGQ_ALLOC:
        {
            MsgqMsg msg ;
            retStatus = PMGR_MSGQ_alloc (args->apiArgs.msgqAllocArgs.poolId,
                                         args->apiArgs.msgqAllocArgs.size,
                                         &msg) ;
            args->apiArgs.msgqAllocArgs.msgAddr = (Uint32) msg ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_MSGQ_FREE:
        {
            retStatus = PMGR_MSGQ_free (args->apiArgs.msgqFreeArgs.msg) ;
            args->apiStatus = retStatus ;
        }
        break ;
#endif /* if defined (MSGQ_COMPONENT) */

    case CMD_PROC_SETUP:
        {
#if defined (MSGQ_COMPONENT)
            LINKCFG_Gpp * gppObj ;
#endif /* if defined (MSGQ_COMPONENT) */

            retStatus = PMGR_PROC_setup (args->apiArgs.procSetupArgs.linkCfg) ;

#if defined (MSGQ_COMPONENT)
            if (DSP_SUCCEEDED (retStatus)) {
                gppObj = LDRV_LinkCfgPtr->gppObject ;
                DRV_MaxMsgqs = gppObj->maxMsgqs ;
            }
#endif /* if defined (MSGQ_COMPONENT) */

            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_PROC_DESTROY:
        {
            retStatus = PMGR_PROC_destroy () ;
            args->apiStatus = retStatus ;
#if defined (DDSP_DEBUG)
            MEM_Debug () ;
#endif /* if defined (DDSP_DEBUG) */
        }
        break ;

    case CMD_PROC_ISLASTDESTROY:
        {
            args->apiArgs.procIsLastDestroyArgs.lastDestroy =
                          (PMGR_PROC_getSetupRefCount () == 1) ? TRUE : FALSE ;
            args->apiStatus = DSP_SOK ;
        }
        break ;

    case CMD_PROC_ISLASTDETACH:
        {
            args->apiArgs.procIsLastDetachArgs.lastDetach =
                        (PMGR_PROC_getAttachRefCount (
                             args->apiArgs.procIsLastDetachArgs.procId) == 1) ?
                                    TRUE : FALSE ;
            args->apiStatus = DSP_SOK ;
        }
        break ;

#if defined (MPCS_COMPONENT)
    case CMD_MPCS_MAPREGION:
        {
            retStatus = LDRV_MPCS_getMemInfo (
                                   &args->apiArgs.mpcsMapArgs.mpcsRegionArgs) ;
            args->apiStatus = retStatus ;
        }
        break ;
#endif /* if defined (MPCS_COMPONENT) */

#if defined (RINGIO_COMPONENT)
    case CMD_RINGIO_MAPREGION:
        {
            retStatus = LDRV_RINGIO_getMemInfo (
                                   &args->apiArgs.ringIoArgs.ringioRegionArgs) ;
            args->apiStatus = retStatus ;
        }
        break ;
#endif /* if defined (RINGIO_COMPONENT) */

#if defined (MPLIST_COMPONENT)
    case CMD_MPLIST_MAPREGION:
        {
            retStatus = LDRV_MPLIST_getMemInfo (
                                   &args->apiArgs.mplistArgs.mplistRegionArgs) ;
            args->apiStatus = retStatus ;
        }
        break ;
#endif /* if defined (MPLIST_COMPONENT) */

    case CMD_PROC_START:
        retStatus = PMGR_PROC_start (args->apiArgs.procStartArgs.procId) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_PROC_STOP:
        retStatus = PMGR_PROC_stop (args->apiArgs.procStopArgs.procId) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_PROC_GETSYMBOLADDRESS:
        hr = CeOpenCallerBuffer((PVOID*) &imgMapPtr, 
            args->apiArgs.procGetSymbolAddressArgs.symbolName,
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
        
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.procGetSymbolAddressArgs.dspAddr,
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
        
        retStatus = PMGR_PROC_getSymbolAddress (
                        args->apiArgs.procGetSymbolAddressArgs.procId,
                        imgMapPtr,
                        args->apiArgs.procGetSymbolAddressArgs.dspAddr) ;
        
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.procGetSymbolAddressArgs.dspAddr, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
        
        hr = CeCloseCallerBuffer((PVOID) imgMapPtr, 
            args->apiArgs.procGetSymbolAddressArgs.symbolName, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
        
        args->apiStatus = retStatus ;
        break ;

    case CMD_PROC_LOAD:
        hr = CeOpenCallerBuffer((PVOID*) &imgMapPtr, 
            args->apiArgs.procLoadArgs.imagePath, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
        if (args->apiArgs.procLoadArgs.argv != NULL) {
            hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                args->apiArgs.procLoadArgs.argv, MAX_ARGUMENT_VAL_LENGTH, 
                ARG_IO_PTR, FALSE);
        }
        if (args->apiArgs.procLoadArgs.argc > 0) { 
            /* Perform a Mapping of each ARGV Argument Pointers */
            for (argCount = 0; argCount <  args->apiArgs.procLoadArgs.argc; argCount++) {
                if (((Char8 **)argMapPtr)[argCount] == NULL) 
                    break;

                hr = CeOpenCallerBuffer((PVOID*) &((Char8 **)argMapPtr)[argCount], 
                    args->apiArgs.procLoadArgs.argv[argCount],
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
            }
        }
        retStatus = PMGR_PROC_load (args->apiArgs.procLoadArgs.procId,
            imgMapPtr, args->apiArgs.procLoadArgs.argc, argMapPtr) ;
                                    
        args->apiStatus = retStatus ;
        if (args->apiArgs.procLoadArgs.argc > 0) { 
            /* Perform a Unmapping for each of the ARGV Argument Pointers */
            for (argCount = 0; argCount <  args->apiArgs.procLoadArgs.argc; argCount++) {
                if (((Char8 **)argMapPtr)[argCount] == NULL) 
                    break;
                
                hr = CeCloseCallerBuffer((PVOID) ((Char8 **)argMapPtr)[argCount], 
                    args->apiArgs.procLoadArgs.argv[argCount], 
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
            }                                  
        }

        if (args->apiArgs.procLoadArgs.argv != NULL) {
            hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                args->apiArgs.procLoadArgs.argv, MAX_ARGUMENT_VAL_LENGTH, 
                ARG_IO_PTR);
        }

        hr = CeCloseCallerBuffer((PVOID) imgMapPtr, 
            args->apiArgs.procLoadArgs.imagePath, MAX_ARGUMENT_VAL_LENGTH, 
            ARG_IO_PTR);
        break ;

    case CMD_PROC_LOADSECTION:
        hr = CeOpenCallerBuffer((PVOID*) &imgMapPtr, 
            args->apiArgs.procLoadSectionArgs.imagePath,
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
        retStatus = PMGR_PROC_loadSection (
            args->apiArgs.procLoadSectionArgs.procId,
            imgMapPtr,args->apiArgs.procLoadSectionArgs.sectID) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) imgMapPtr, 
            args->apiArgs.procLoadSectionArgs.imagePath, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
        break ;

    case CMD_PROC_READ:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.procReadArgs.buffer, 
            args->apiArgs.procReadArgs.numBytes, ARG_IO_PTR, FALSE);
        retStatus = PMGR_PROC_read (args->apiArgs.procReadArgs.procId,
                                    args->apiArgs.procReadArgs.dspAddr,
                                    args->apiArgs.procReadArgs.numBytes,
						            argMapPtr) ;

        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.procReadArgs.buffer, 
            args->apiArgs.procReadArgs.numBytes, ARG_IO_PTR);
        args->apiStatus = retStatus ;
        break ;

    case CMD_PROC_WRITE:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.procWriteArgs.buffer, 
            args->apiArgs.procWriteArgs.numBytes, ARG_IO_PTR, FALSE);
        retStatus = PMGR_PROC_write (args->apiArgs.procWriteArgs.procId,
                                     args->apiArgs.procWriteArgs.dspAddr,
                                     args->apiArgs.procWriteArgs.numBytes,
						            argMapPtr) ;
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.procWriteArgs.buffer, 
            args->apiArgs.procWriteArgs.numBytes, ARG_IO_PTR);
        args->apiStatus = retStatus ;
        break ;

    case CMD_PROC_ATTACH:
        {
#if defined (CHNL_COMPONENT)
            ProcessorId   procId ;
            ChannelId     chnlId ;
#endif /* if defined (CHNL_COMPONENT) */
            if (args->apiArgs.procAttachArgs.attr != NULL) {
                hr = CeOpenCallerBuffer((PVOID*) &attr, 
                    args->apiArgs.procAttachArgs.attr, 
                    sizeof (ProcAttr), ARG_IO_PTR, FALSE);
            }
            else {
                attr = NULL ;
            }            
            retStatus = PMGR_PROC_attach (args->apiArgs.procAttachArgs.procId,
                                          attr,
                                          args->apiArgs.procAttachArgs.prcsInfo) ;
#if defined (CHNL_COMPONENT)
            if ((DSP_SUCCEEDED (retStatus))
                && (retStatus != DSP_SALREADYATTACHED)) 
            {
                procId = args->apiArgs.procAttachArgs.procId ;
                for (chnlId = 0 ; (   (chnlId < MAX_CHANNELS)
                                   && DSP_SUCCEEDED (retStatus)); chnlId++) {
                    retStatus = LDRV_DATA_getPoolId (
                                    procId,
                                    chnlId,
                                    &DRV_ChnlIdToPoolId [procId][chnlId]) ;
                    if (retStatus == DSP_ENOTFOUND) {
                        /* DSP_ENOTFOUND indicates that the channel ID is
                         * not in the range of any configured data driver.
                         */
                        retStatus = DSP_SOK ;
                    }
                }
            }
            if ((DSP_SUCCEEDED (retStatus)) && (args->apiArgs.procAttachArgs.map != NULL))
            {
                tabSize = sizeof (Uint32) * MAX_DSPS * MAX_CHANNELS ;
                hr = CeOpenCallerBuffer((PVOID*)&argMapPtr, 
                                            args->apiArgs.procAttachArgs.map, 
                                            tabSize, ARG_IO_PTR, FALSE);
                if (hr == S_OK)
                {
                    memcpy(argMapPtr, DRV_ChnlIdToPoolId, tabSize);
                    hr = CeCloseCallerBuffer((PVOID)argMapPtr, 
                                            args->apiArgs.procAttachArgs.map,
                                             tabSize, ARG_IO_PTR);
                }
                else
                {
                    retStatus = DSP_EPOINTER;                    
                }
            }
#endif /* if defined (CHNL_COMPONENT) */

            args->apiStatus = retStatus ;
            if (args->apiArgs.procAttachArgs.attr != NULL) {
                hr = CeCloseCallerBuffer((PVOID) attr, 
                    args->apiArgs.procAttachArgs.attr, 
                    sizeof (ProcAttr), ARG_IO_PTR);
            }
        }
        break ;

    case CMD_PROC_DETACH:
        retStatus = PMGR_PROC_detach (args->apiArgs.procDetachArgs.procId,
                                      args->apiArgs.procDetachArgs.prcsInfo) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_PROC_CONTROL:
        /* although currently arg is a ProcMemMapInfo, but use generic size here 
           for future expandibility */
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.procControlArgs.arg, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
        retStatus = PMGR_PROC_control (
                     args->apiArgs.procControlArgs.procId,
                     args->apiArgs.procControlArgs.cmd,
                     argMapPtr) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.procControlArgs.arg, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
        break ;

    case CMD_PROC_GETSTATE:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.procGetStateArgs.procState, 
            sizeof(PROC_State), ARG_IO_PTR, FALSE);
        
        retStatus = PMGR_PROC_getState (
                        args->apiArgs.procGetStateArgs.procId,
                        args->apiArgs.procGetStateArgs.procState) ;
        args->apiStatus = retStatus ;

        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.procGetStateArgs.procState, 
            sizeof(PROC_State), ARG_IO_PTR);
        break ;

#if defined (DDSP_PROFILE)
    case CMD_PROC_INSTRUMENT:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.procInstrumentArgs.procStats, 
            sizeof(PROC_Instrument), ARG_IO_PTR, FALSE);
        retStatus = PMGR_PROC_instrument (
                      args->apiArgs.procInstrumentArgs.procId,
                      args->apiArgs.procInstrumentArgs.procStats) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.procInstrumentArgs.procStats, 
            sizeof(PROC_Instrument), ARG_IO_PTR);
        break ;
#endif /* if defined (DDSP_PROFILE) */

#if defined (DDSP_DEBUG)
    case CMD_PROC_DEBUG:
        PMGR_PROC_debug (args->apiArgs.procDebugArgs.procId) ;
        break ;
#endif /* if defined (DDSP_DEBUG) */

    case CMD_PROC_CLEANUP:
        DSPLINK_Cleanup () ;
        break ;

#if defined (CHNL_COMPONENT)
    case CMD_CHNL_CREATE:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.chnlCreateArgs.attrs, 
            sizeof (ChannelAttrs), ARG_IO_PTR, FALSE);
        retStatus = PMGR_CHNL_create (
                        args->apiArgs.chnlCreateArgs.procId,
                        args->apiArgs.chnlCreateArgs.chnlId,
                        argMapPtr,
                        NULL) ;

        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.chnlCreateArgs.attrs, 
            sizeof (ChannelAttrs), ARG_IO_PTR);
        args->apiStatus = retStatus ;
        break ;

    case CMD_CHNL_DELETE:
        retStatus = PMGR_CHNL_delete (args->apiArgs.chnlDeleteArgs.procId,
                                      args->apiArgs.chnlDeleteArgs.chnlId,
                                      NULL) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_CHNL_ALLOCATEBUFFER:
    {
        ProcessorId  procId   ;
        ChannelId    chnlId   ;
        Uint32       size     ;
        Uint32       numBufs  ;
        Char8 **     bufArray ;        

        procId   = args->apiArgs.chnlAllocateBufferArgs.procId ;
        chnlId   = args->apiArgs.chnlAllocateBufferArgs.chnlId ;
        size     = args->apiArgs.chnlAllocateBufferArgs.size ;
        numBufs  = args->apiArgs.chnlAllocateBufferArgs.numBufs ;
        bufArray = args->apiArgs.chnlAllocateBufferArgs.bufArray ;

        hr = CeOpenCallerBuffer((PVOID*) &bufArray, 
            args->apiArgs.chnlAllocateBufferArgs.bufArray,
            (sizeof (Char8 *) * numBufs), ARG_IO_PTR, FALSE);       
        
        retStatus = PMGR_CHNL_allocateBuffer (procId,
                                              chnlId,
                                              bufArray,
                                              size,
                                              numBufs,
                                              NULL) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) bufArray, 
            args->apiArgs.chnlAllocateBufferArgs.bufArray, 
            (sizeof (Char8 *) * numBufs), ARG_IO_PTR);
    }
    break ;

    case CMD_CHNL_FREEBUFFER:
    {
        ProcessorId  procId   ;
        ChannelId    chnlId   ;
        Uint32       numBufs  ;
        Char8 **     bufArray ;

        procId   = args->apiArgs.chnlFreeBufferArgs.procId ;
        chnlId   = args->apiArgs.chnlFreeBufferArgs.chnlId ;
        numBufs  = args->apiArgs.chnlFreeBufferArgs.numBufs ;
        hr = CeOpenCallerBuffer((PVOID*) &bufArray, 
            args->apiArgs.chnlFreeBufferArgs.bufArray,
            (sizeof (Uint32) * numBufs), ARG_IO_PTR, FALSE);

        retStatus = PMGR_CHNL_freeBuffer (procId,
                                          chnlId,
                                          bufArray,
                                          numBufs,
                                          NULL) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) bufArray, 
            args->apiArgs.chnlFreeBufferArgs.bufArray, 
            (sizeof (Uint32) * numBufs), ARG_IO_PTR);
    }
    break ;

    case CMD_CHNL_IDLE:
        retStatus = PMGR_CHNL_idle (
                                args->apiArgs.chnlIdleArgs.procId,
                                args->apiArgs.chnlIdleArgs.chnlId,
                                NULL) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_CHNL_FLUSH:
        retStatus = PMGR_CHNL_flush (
                                args->apiArgs.chnlFlushArgs.procId,
                                args->apiArgs.chnlFlushArgs.chnlId,
                                NULL) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_CHNL_CONTROL:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.chnlControlArgs.arg, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
        retStatus = PMGR_CHNL_control (
                        args->apiArgs.chnlControlArgs.procId,
                        args->apiArgs.chnlControlArgs.chnlId,
                        args->apiArgs.chnlControlArgs.cmd,
                        argMapPtr) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.chnlControlArgs.arg, 
            MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
        break ;

#if defined (DDSP_PROFILE)
    case CMD_CHNL_INSTRUMENT:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
            args->apiArgs.chnlInstrumentArgs.chnlStats, 
            sizeof(CHNL_Instrument), ARG_IO_PTR, FALSE);
        retStatus = PMGR_CHNL_instrument (
                       args->apiArgs.chnlInstrumentArgs.procId,
                       args->apiArgs.chnlInstrumentArgs.chnlId,
                       args->apiArgs.chnlInstrumentArgs.chnlStats) ;
        args->apiStatus = retStatus ;
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
            args->apiArgs.chnlInstrumentArgs.chnlStats, 
            sizeof(CHNL_Instrument), ARG_IO_PTR);
        break ;
#endif /* if defined (DDSP_PROFILE) */

#if defined (DDSP_DEBUG)
    case CMD_CHNL_DEBUG:
        PMGR_CHNL_debug (args->apiArgs.chnlDebugArgs.procId,
                         args->apiArgs.chnlDebugArgs.chnlId) ;
        break ;
#endif /* if defined (DDSP_DEBUG) */

    case CMD_DRV_GETCHNLMAPTABLE_ADDRESS:
        args->apiArgs.drvPhyAddrArgs.phyAddr
                                = (Void *) DRV_MemAllocAttrs.physicalAddress ;
        break ;    
#endif /* if defined (CHNL_COMPONENT) */

#if defined (MSGQ_COMPONENT)
    case CMD_MSGQ_TRANSPORTOPEN:
        {
            Pvoid attrs = NULL;
            /* use generic value - MAX_ARGUMENT_VAL_LENGTH */
            hr = CeOpenCallerBuffer((PVOID*) &attrs, 
                                    args->apiArgs.msgqTransportOpenArgs.attrs, 
                                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
            retStatus = PMGR_MSGQ_transportOpen (args->apiArgs.msgqTransportOpenArgs.procId,
                                                attrs) ;
            args->apiStatus = retStatus ;
            hr = CeCloseCallerBuffer((PVOID) attrs, 
                                    args->apiArgs.msgqTransportOpenArgs.attrs, 
                                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
        }
        break ;

    case CMD_MSGQ_TRANSPORTCLOSE:
        retStatus = PMGR_MSGQ_transportClose (
                                  args->apiArgs.msgqTransportCloseArgs.procId) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_MSGQ_OPEN:
        /* Only NULL attributes are supported for Wince for MSGQ_open (). */
        if (args->apiArgs.msgqOpenArgs.attrs != NULL) {
            retStatus = DSP_ENOTSUPPORTED ;
        }
        else {

            Pstr                queueName = NULL ;

            MSGQ_Queue          *msgqQueue ;
            if (args->apiArgs.msgqOpenArgs.queueName != NULL) {                
                hr = CeOpenCallerBuffer((PVOID*) &queueName, 

                    args->apiArgs.msgqOpenArgs.queueName, 

                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);

            }

            hr = CeOpenCallerBuffer((PVOID*) &msgqQueue, 
                args->apiArgs.msgqOpenArgs.msgqQueue, 
                sizeof(MSGQ_Queue), ARG_IO_PTR, FALSE);
            retStatus = PMGR_MSGQ_open (queueName, msgqQueue,
                                        args->apiArgs.msgqOpenArgs.attrs,
                                        args->apiArgs.msgqOpenArgs.prcsInfo) ;
            hr = CeCloseCallerBuffer((PVOID) msgqQueue, 
                args->apiArgs.msgqOpenArgs.msgqQueue, 
                sizeof(MSGQ_Queue), ARG_IO_PTR);

            if (queueName!=NULL) {

                hr = CeCloseCallerBuffer((PVOID) queueName, 
                    args->apiArgs.msgqOpenArgs.queueName, 

                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);

            }

        }
        args->apiStatus = retStatus ;
        break ;

    case CMD_MSGQ_CLOSE:
        retStatus = PMGR_MSGQ_close (args->apiArgs.msgqCloseArgs.msgqQueue,
                                     args->apiArgs.msgqCloseArgs.prcsInfo) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_MSGQ_LOCATE:
        {
            Pstr                queueName ;
            MSGQ_Queue          *msgqQueue ;
            MSGQ_LocateAttrs    *attrs ;
            hr = CeOpenCallerBuffer((PVOID*) &queueName, 
                args->apiArgs.msgqLocateArgs.queueName, 
                MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);

            hr = CeOpenCallerBuffer((PVOID*) &msgqQueue, 
                args->apiArgs.msgqLocateArgs.msgqQueue, 
                sizeof(MSGQ_Queue), ARG_IO_PTR, FALSE);
        
            hr = CeOpenCallerBuffer((PVOID*) &attrs, 
                args->apiArgs.msgqLocateArgs.attrs, 
                sizeof(MSGQ_LocateAttrs), ARG_IO_PTR, FALSE);
            retStatus = PMGR_MSGQ_locate (queueName, msgqQueue, attrs) ;

            hr = CeCloseCallerBuffer((PVOID) attrs, 
                args->apiArgs.msgqLocateArgs.attrs, 
                MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);

            hr = CeCloseCallerBuffer((PVOID) msgqQueue, 
                args->apiArgs.msgqLocateArgs.msgqQueue, 
                sizeof(MSGQ_Queue), ARG_IO_PTR);

            hr = CeCloseCallerBuffer((PVOID) queueName, 
                args->apiArgs.msgqLocateArgs.queueName, 
                sizeof(MSGQ_LocateAttrs), ARG_IO_PTR);
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_MSGQ_LOCATEASYNC:
        {
            Pstr                    queueName ;
            MSGQ_LocateAsyncAttrs    *attrs ;

            hr = CeOpenCallerBuffer((PVOID*) &queueName, 
                args->apiArgs.msgqLocateAsyncArgs.queueName,
                MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);

            hr = CeOpenCallerBuffer((PVOID*) &attrs, 
                args->apiArgs.msgqLocateAsyncArgs.attrs, 
                sizeof(MSGQ_LocateAsyncAttrs), ARG_IO_PTR, FALSE);

            /* not sure if anyone uses the arg pointer; should we marshall it here??? */
            
            retStatus = PMGR_MSGQ_locateAsync (
                                       queueName,
                                       args->apiArgs.msgqLocateAsyncArgs.replyQueue,
                                       attrs) ;
                                       
            hr = CeCloseCallerBuffer((PVOID) attrs, 
                args->apiArgs.msgqLocateAsyncArgs.attrs, 
                sizeof(MSGQ_LocateAsyncAttrs), ARG_IO_PTR);

            hr = CeCloseCallerBuffer((PVOID) queueName, 
                args->apiArgs.msgqLocateAsyncArgs.queueName, 
                MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);

            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_MSGQ_RELEASE:
        retStatus = PMGR_MSGQ_release (
                                   args->apiArgs.msgqReleaseArgs.msgqQueue) ;
        args->apiStatus = retStatus ;
        break ;

    case CMD_MSGQ_COUNT:
        hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                args->apiArgs.msgqCountArgs.count,
                sizeof(Uint16), ARG_IO_PTR, FALSE);
        retStatus = PMGR_MSGQ_count (args->apiArgs.msgqCountArgs.msgqQueue,
                                     args->apiArgs.msgqCountArgs.count) ;
        hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                args->apiArgs.msgqCountArgs.count, 
                sizeof(Uint16), ARG_IO_PTR);
        args->apiStatus = retStatus ;
        break ;

    case CMD_MSGQ_SETERRORHANDLER:
        retStatus = PMGR_MSGQ_setErrorHandler (
                               args->apiArgs.msgqSetErrorHandlerArgs.errorQueue,
                               args->apiArgs.msgqSetErrorHandlerArgs.poolId) ;
        args->apiStatus = retStatus ;
        break ;

#if defined (DDSP_PROFILE)
    case CMD_MSGQ_INSTRUMENT:
        {            
            hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                    args->apiArgs.msgqInstrumentArgs.retVal,
                    sizeof(MSGQ_Instrument), ARG_IO_PTR, FALSE);
            retStatus = PMGR_MSGQ_instrument (
                                         args->apiArgs.msgqInstrumentArgs.msgqQueue,
                                         args->apiArgs.msgqInstrumentArgs.retVal) ;
            args->apiStatus = retStatus ;
            hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                args->apiArgs.msgqInstrumentArgs.retVal, 
                sizeof(MSGQ_Instrument), ARG_IO_PTR);
        }
        break ;
#endif /* if defined (DDSP_PROFILE) */

#if defined (DDSP_DEBUG)
    case CMD_MSGQ_DEBUG:
        PMGR_MSGQ_debug (args->apiArgs.msgqDebugArgs.msgqQueue) ;
        args->apiStatus = retStatus ;
        break ;
#endif /* if defined (DDSP_DEBUG) */

#endif /* if defined (MSGQ_COMPONENT) */

#if defined (POOL_COMPONENT)
    case CMD_POOL_ALLOC:
        {
            retStatus = LDRV_POOL_alloc (args->apiArgs.poolAllocArgs.poolId,
                                         args->apiArgs.poolAllocArgs.bufPtr,
                                         args->apiArgs.poolAllocArgs.size) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_POOL_FREE:
        {
            retStatus = LDRV_POOL_free (args->apiArgs.poolFreeArgs.poolId,
                                        args->apiArgs.poolFreeArgs.bufPtr,
                                        args->apiArgs.poolFreeArgs.size) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_POOL_OPEN:
        {
            POOL_OpenParams * poolOpenParams = (POOL_OpenParams *)NULL ;
#if defined ZCPY_LINK
            SMAPOOL_Attrs * poolParams  =  NULL ;
#endif
#if defined (PCPY_LINK)
            BUFPOOL_Attrs * poolParams  =  NULL ;
#endif
            /* 
             * If POOL_OPEN Parameters are null, then
             * return DSP_EINVALIDARG to the API
             * Layer
             */
            if (args->apiArgs.poolOpenArgs.params == NULL) {
                args->apiStatus = DSP_EINVALIDARG ;                                       
            }
            else {
                hr = CeOpenCallerBuffer((PVOID*) &poolOpenParams, 
                    args->apiArgs.poolOpenArgs.params,
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);

                /* NULL params are OK */
                if (poolOpenParams->params != NULL) {

#if defined (ZCPY_LINK) || defined (PCPY_LINK)
                    hr = CeOpenCallerBuffer((PVOID*) &poolParams, 
                        poolOpenParams->params, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
#endif
                                  
#if defined ZCPY_LINK
                    hr = CeOpenCallerBuffer((PVOID*) &poolParams->bufSizes, 
                        ((SMAPOOL_Attrs *)poolOpenParams->params)->bufSizes, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);                    
#endif
#if defined (PCPY_LINK)
                    hr = CeOpenCallerBuffer((PVOID*) &poolParams->bufSizes, 
                        ((BUFPOOL_Attrs *)poolOpenParams->params)->bufSizes, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);                    
#endif

#if defined ZCPY_LINK
                    hr = CeOpenCallerBuffer((PVOID*) &poolParams->numBuffers, 
                        ((SMAPOOL_Attrs *)poolOpenParams->params)->numBuffers, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);                    
#endif
#if defined PCPY_LINK
                    hr = CeOpenCallerBuffer((PVOID*) &poolParams->numBuffers, 
                        ((BUFPOOL_Attrs *)poolOpenParams->params)->numBuffers, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);                    
#endif
                }



                retStatus = LDRV_POOL_open (args->apiArgs.poolOpenArgs.poolId,
                                            poolOpenParams ) ;
                                        
                if (poolOpenParams->params != NULL) {
#if defined ZCPY_LINK
                    hr = CeCloseCallerBuffer((PVOID) poolParams->numBuffers, 
                        ((SMAPOOL_Attrs *)poolOpenParams->params)->numBuffers, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);                    
#endif
#if defined (PCPY_LINK)
                    hr = CeCloseCallerBuffer((PVOID) poolParams->numBuffers, 
                        ((BUFPOOL_Attrs *)poolOpenParams->params)->numBuffers, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);                    
#endif
#if defined ZCPY_LINK
                    hr = CeCloseCallerBuffer((PVOID) poolParams->bufSizes, 
                        ((SMAPOOL_Attrs *)poolOpenParams->params)->bufSizes, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
#endif
#if defined (PCPY_LINK)
                    hr = CeCloseCallerBuffer((PVOID) poolParams->bufSizes, 
                        ((BUFPOOL_Attrs *)poolOpenParams->params)->bufSizes, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
#endif

#if defined (ZCPY_LINK) || defined (PCPY_LINK)
                    hr = CeCloseCallerBuffer((PVOID) poolParams, 
                        poolOpenParams->params, 
                        MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
#endif
                }
                hr = CeCloseCallerBuffer((PVOID) poolOpenParams, 
                    args->apiArgs.poolOpenArgs.params, 
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_POOL_CLOSE:
        {
            retStatus = LDRV_POOL_close (args->apiArgs.poolCloseArgs.poolId) ;

            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_POOL_WRITEBACK:
        {
            retStatus = LDRV_POOL_writeback (args->apiArgs.poolWBArgs.poolId,
                                             args->apiArgs.poolWBArgs.bufPtr,
                                             args->apiArgs.poolWBArgs.size) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_POOL_INVALIDATE:
        {
            retStatus = LDRV_POOL_invalidate (args->apiArgs.poolInvArgs.poolId,
                                              args->apiArgs.poolInvArgs.bufPtr,
                                              args->apiArgs.poolInvArgs.size) ;
            args->apiStatus = retStatus ;
        }
        break ;
#endif /* if defined (POOL_COMPONENT) */

#if defined (NOTIFY_COMPONENT)
    case CMD_NOTIFY_INITIALIZE:
        {
            retStatus = NOTIFY_KnlInitialize
                                (args->apiArgs.notifyInitializeArgs.dspId) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_NOTIFY_FINALIZE:
        {
            retStatus = NOTIFY_KnlFinalize
                                (args->apiArgs.notifyFinalizeArgs.dspId) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_NOTIFY_REGISTER:
        {
            if (args->apiArgs.notifyRegisterArgs.cbckArg != NULL) {
                hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                    args->apiArgs.notifyRegisterArgs.cbckArg,
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
            }
            retStatus = NOTIFY_KnlRegister
                                (args->apiArgs.notifyRegisterArgs.dspId,
                                 args->apiArgs.notifyRegisterArgs.ipsId,
                                 args->apiArgs.notifyRegisterArgs.eventNo,
                                 args->apiArgs.notifyRegisterArgs.fnNotifyCbck,
                                 argMapPtr) ;
            args->apiStatus = retStatus ;
            if (args->apiArgs.notifyRegisterArgs.cbckArg != NULL) {
                hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                    args->apiArgs.notifyRegisterArgs.cbckArg, 
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
            }
        }
        break ;

    case CMD_NOTIFY_UNREGISTER:
        {
            if (args->apiArgs.notifyUnregisterArgs.cbckArg != NULL) {
                hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                    args->apiArgs.notifyUnregisterArgs.cbckArg,
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);
            }
            retStatus = NOTIFY_KnlUnregister
                             (args->apiArgs.notifyUnregisterArgs.dspId,
                              args->apiArgs.notifyUnregisterArgs.ipsId,
                              args->apiArgs.notifyUnregisterArgs.eventNo,
                              args->apiArgs.notifyUnregisterArgs.fnNotifyCbck,
                              argMapPtr) ;
            args->apiStatus = retStatus ;
            if (args->apiArgs.notifyUnregisterArgs.cbckArg != NULL) {
                hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                    args->apiArgs.notifyUnregisterArgs.cbckArg, 
                    MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
            }
        }
        break ;

    case CMD_NOTIFY_NOTIFY:
        {
            retStatus = NOTIFY_KnlNotify
                             (args->apiArgs.notifyNotifyArgs.dspId,
                              args->apiArgs.notifyNotifyArgs.ipsId,
                              args->apiArgs.notifyNotifyArgs.eventNo,
                              args->apiArgs.notifyNotifyArgs.payload) ;
            args->apiStatus = retStatus ;
        }
        break ;

#if defined (DDSP_PROFILE)
    case CMD_NOTIFY_INSTRUMENT:
        {
            hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                                    args->apiArgs.ipsInstrumentArgs.ipsStats,
                                    sizeof(IPS_Instrument), ARG_IO_PTR, FALSE);
            retStatus = NOTIFY_KnlInstrument
                                (args->apiArgs.ipsInstrumentArgs.dspId,
                                 args->apiArgs.ipsInstrumentArgs.ipsId,
                                 args->apiArgs.ipsInstrumentArgs.ipsStats) ;
            args->apiStatus = retStatus ;
            hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                                     args->apiArgs.ipsInstrumentArgs.ipsStats, 
                                     sizeof(IPS_Instrument), ARG_IO_PTR);
        }
        break ;
#endif /* if defined (DDSP_PROFILE) */
#endif /* #if defined (NOTIFY_COMPONENT) */

    case CMD_IDM_INIT:
        {
            retStatus = IDM_init () ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_IDM_EXIT:
        {
            retStatus = IDM_exit () ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_IDM_CREATE:
        {
            hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                args->apiArgs.idmCreateArgs.attrs,
                sizeof(IDM_Attrs), ARG_IO_PTR, FALSE);
            retStatus = IDM_create (args->apiArgs.idmCreateArgs.key,
                                    argMapPtr) ;

            hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                args->apiArgs.idmCreateArgs.attrs, 
                sizeof(IDM_Attrs), ARG_IO_PTR);
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_IDM_DELETE:
        {
            retStatus = IDM_delete (args->apiArgs.idmDeleteArgs.key) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_IDM_ACQUIREID:
        {
            hr = CeOpenCallerBuffer((PVOID*) &imgMapPtr, 
                args->apiArgs.idmAcquireIdArgs.idKey,
                MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR, FALSE);

            hr = CeOpenCallerBuffer((PVOID*) &argMapPtr, 
                args->apiArgs.idmAcquireIdArgs.id,
                sizeof(Uint32), ARG_IO_PTR, FALSE);

            retStatus = IDM_acquireId (args->apiArgs.idmAcquireIdArgs.key,
                                       imgMapPtr,
                                       argMapPtr) ;

            hr = CeCloseCallerBuffer((PVOID) argMapPtr, 
                args->apiArgs.idmAcquireIdArgs.id, 
                sizeof(Uint32), ARG_IO_PTR);

            hr = CeCloseCallerBuffer((PVOID) imgMapPtr, 
                args->apiArgs.idmAcquireIdArgs.idKey, 
                MAX_ARGUMENT_VAL_LENGTH, ARG_IO_PTR);
                args->apiStatus = retStatus ;
            }
        }
        break ;

    case CMD_IDM_RELEASEID:
        {
            retStatus = IDM_releaseId (args->apiArgs.idmReleaseIdArgs.key,
                                       args->apiArgs.idmReleaseIdArgs.id) ;
            args->apiStatus = retStatus ;
        }
        break ;

    case CMD_KERN_MAP_IO_SPACE:
        {
            PHYSICAL_ADDRESS PhysicalIoBase;

            PhysicalIoBase.LowPart = (DWORD) args->apiArgs.kernMapIoSpaceArgs.PhysicalAddress;
            PhysicalIoBase.HighPart = 0;

            args->apiArgs.kernMapIoSpaceArgs.retVal = MmMapIoSpace(PhysicalIoBase, 
                (ULONG)args->apiArgs.kernMapIoSpaceArgs.NumberOfBytes,
                args->apiArgs.kernMapIoSpaceArgs.CacheEnable);


            args->apiStatus = DSP_SOK ;
        }
        break ;

    case CMD_KERN_UNMAP_IO_SPACE:
        {
            PHYSICAL_ADDRESS PhysicalIoBase;


            MmUnmapIoSpace((PVOID)args->apiArgs.kernUnmapIoSpaceArgs.BaseAddress,
                (ULONG)args->apiArgs.kernUnmapIoSpaceArgs.NumberOfBytes);

            args->apiStatus = DSP_SOK ;
        }
        break ;

    case CMD_KERN_MAP_PHYS_TO_USER:
        {
            LPVOID  pVirtualAddress;
            BOOL    bSuccess;
            HANDLE  callerProcess=GetCallerProcess (); 
            // need to use this since VirtualAllocEx and VirtualCopyEx needs HANDLE and not DWORD 
            // as returned by GetDirectCallerProcessID

            pVirtualAddress = VirtualAllocEx(callerProcess, 0, 
                args->apiArgs.kernMapPhysToUser.size, MEM_RESERVE, 
                PAGE_NOACCESS);

            if (pVirtualAddress != NULL)   {
                bSuccess = VirtualCopyEx(
                    callerProcess, pVirtualAddress, GetCurrentProcess(),
                    (PVOID)(args->apiArgs.kernMapPhysToUser.PhysicalAddress>>8), 
                    args->apiArgs.kernMapPhysToUser.size, 
                    ((args->apiArgs.kernMapPhysToUser.CacheEnable == FALSE) ? PAGE_NOCACHE : 0) | 
                    PAGE_READWRITE | PAGE_PHYSICAL );

                if (bSuccess) {
                    args->apiArgs.kernMapPhysToUser.UserAddress = (Uint32)pVirtualAddress;

                } else {
                }
            }
            else
            {
            }
        }
        break;

    case CMD_KERN_UNMAP_PHYS_TO_USER:
        {
            BOOL    bSuccess;

            bSuccess = VirtualFreeEx(GetCallerProcess(), 
                (PVOID)args->apiArgs.kernUnmapPhysToUser.UserAddress,
                0, MEM_RELEASE);

            if (bSuccess) {
            } else {
            }
        }
        break;
    default:
        TRC_1PRINT (TRC_LEVEL7,
                    "Incorrect command id specified [0x%x]\n",
                    cmd) ;
        status = DSP_EFAIL ;
        break ;
    }


    TRC_1LEAVE ("DRV_CallAPI", status) ;

    return status ;
}


/** ----------------------------------------------------------------------------
 *  @func  DSPLINK_Cleanup
 *
 *  @desc  Clean up the kernel driver.
 *  ----------------------------------------------------------------------------
 */
STATIC
NORMAL_API
Void
DSPLINK_Cleanup (Void)
{
#if defined (MSGQ_COMPONENT)
    DSP_STATUS  status = DSP_SOK ;
    MSGQ_Queue  msgqQueue ;
    Uint32      i ;

    /* Close any open Message Queues */
    for (i = 0 ; i < DRV_MaxMsgqs ; i++) {
        msgqQueue = ((Uint32) ID_GPP << 16) | i ;
        status = PMGR_MSGQ_close (msgqQueue, NULL) ;

        if (DSP_SUCCEEDED (status)) {
            TRC_0PRINT (TRC_LEVEL4, "PMGR_MSGQ_close done\n") ;
        }
    }
#endif /* if defined (MSGQ_COMPONENT) */
}

/** ============================================================================
 *  @func  DllEntry
 *
 *  @desc  Device driver DLL entry point. Any specific intialization is done
 *         here. The debug zones and GT are initialized here. The Successful
 *         loading of the DLL depends upon the retun value of any initialization.
 * 
 *  ============================================================================
 */ 
BOOL WINAPI DllEntry(HINSTANCE hDllInstance, INT nReason,
                         LPVOID lpvReserved)
{
    BOOL bReturnStatus = TRUE;

    TRC_0ENTER("LINK: +DllEntry\n");

    switch (nReason) {
        case DLL_PROCESS_ATTACH:
            DisableThreadLibraryCalls((HMODULE)hDllInstance);
            TRC_0PRINT(TRC_LEVEL2,"LINK: DllEntry DLL_PROCESS_ATTACH\n");
            break;

        case DLL_PROCESS_DETACH:
            TRC_0PRINT(TRC_LEVEL2,"LINK: DllEntry DLL_PROCESS_DETACH\n");
            break;

        case DLL_THREAD_ATTACH:
            TRC_0PRINT(TRC_LEVEL2,"LINK: DllEntry DLL_THREAD_ATTACH\n");
            break;

        case DLL_THREAD_DETACH:
            TRC_0PRINT(TRC_LEVEL2,"LINK: DllEntry DLL_THREAD_DETACH\n");
            break;

    }
    TRC_0LEAVE("LINK: -DllEntry\n");

    return (bReturnStatus);
}

/** ----------------------------------------------------------------------------
 *  @name   DBL_Init
 *
 *  @desc   Device Manager calls this function as a result of a call 
 *          to the ActivateDeviceEx function. When the user starts using 
 *          a device, Device Manager calls this function to initialize 
 *          the device. Applications do not call this function. 
 *
 *  @arg    LPCTSTR pContext
 *              The component and subcomponent map
 *
 *          LPCVOID lpvBusContext
 *              The Pointer to the Bus Handle
 *
 *  @ret    DWORD
 *              Non-zero to denote operation successfully completed.
 *          0 
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
DWORD
DBL_Init(LPCTSTR pContext, LPCVOID lpvBusContext) 
{
    int drvInit ; 
    DSP_STATUS      status = DSP_SOK ;
    DRV_OBJECT *    hDrvObject = (DRV_OBJECT *)&hDrvObj;
    

    hDrvObject->initialized = FALSE;
    hDrvObject->refCount = 0;

    /* Perform the core initialization */ 
    drvInit = DRV_InitializeModule (pContext);
    if (drvInit == -1) {
        hDrvObject = NULL;
    }
    else {
        hDrvObject->initialized = TRUE;
        hDrvObject->signature = SIGNATURE;    
        hDrvObject->traceMask = 0xFFFF;    /* For the initial setup */
        hDrvObject->refCount = 0;
    }
    

    /* Return the DrvObject */
    return ((DWORD)hDrvObject);
}

/** ----------------------------------------------------------------------------
 *  @name   DBL_Deinit
 *
 *  @desc   This is the de-initialization function for the PMGR Stream 
 *          Interface driver. Device manager invokes this routine.
 *
 *  @arg    DWORD hDeviceContext
 *              Pointer to the Device Driver Context Object.
 *
 *  @ret    DSP_SOK
 *              Operation successfully completed.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
BOOL 
DBL_Deinit (DWORD hDeviceContext) 
{
    DRV_OBJECT *    hDrvObject = (DRV_OBJECT *)hDeviceContext;
    
    
    if (hDrvObject->initialized == TRUE) {
        if (hDrvObject->refCount > 0)
{
            /* cannot free since somebody is still using it */
            return (FALSE);
}
        else
        {                
            DRV_FinalizeModule ();                       
        hDrvObject->signature = SIGNATURE;    
        hDrvObject->traceMask = 0xFFFF;    /* For the initial setup */
        hDrvObject->initialized = FALSE;
        hDrvObject->refCount = 0;
    }     
    }     
    return (TRUE);
}

/** ----------------------------------------------------------------------------
 *  @name   DBL_Open
 *
 *  @desc   This function is used to open the PMGR module for further use. 
 *          This routine will be internally called when the DSP BIOS LINK 
 *          API component performs a call to CreateFile() to obtain a 
 *          handle to a PMGR driver.
 *
 *  @arg    DWORD hDeviceContext
 *              Pointer to the Device Driver Context Object.
 *          DWORD AccessCode
 *              Access Code for the Driver. This argument is not used.
 *          DWORD ShareMode
 *              Open access mode for the Driver.
 *
 *  @ret    DWORD
 *              Operation successfully completed.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
DWORD
DBL_Open (DWORD hDeviceContext, DWORD AccessCode, DWORD ShareMode ) 
{
    DRV_OBJECT *    hDrvObject = (DRV_OBJECT *)hDeviceContext;
        

    if (hDrvObject != NULL) {
        hDrvObject->refCount++;
    }
    

    return (hDeviceContext);
}

/** ----------------------------------------------------------------------------
 *  @name   DBL_Close
 *
 *  @desc   This routine is used to close the handle to the PMGR driver. 
 *          It will be internally invoked when the DSP BIOS LINK API 
 *          Component performs a call to CloseHandle routine.
 *
 *  @arg    DWORD hDeviceContext
 *              Pointer to the Device Driver Context Object.
 *
 *  @ret    BOOL
 *              Operation successfully completed.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
BOOL
DBL_Close (DWORD hOpenContext)
{
    DRV_OBJECT *    hDrvObject = (DRV_OBJECT *)hOpenContext;


    if (hDrvObject != NULL) {
        hDrvObject->refCount--;
    }
            
    return (TRUE);
}


/** ----------------------------------------------------------------------------
 *  @name   DBL_Read
 *
 *  @desc   This routine is used to perform a read operation.
 *
 *  @arg    DWORD hOpenContext
 *              Device Driver Context Object
 *          LPVOID pBuffer
 *              Pointer to the Buffer
 *          DWORD Count
 *              Count of bytes
 *
 *  @ret    DWORD
 *              Number of bytes returned
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
DWORD
DBL_Read (DWORD hOpenContext, LPVOID pBuffer, DWORD Count )
{
    DWORD bytesRead = 0;

#if defined (NOTIFY_COMPONENT)
    bytesRead = UEVENT_GetBuf ((Pvoid) pBuffer) ;
#endif /* #if defined (NOTIFY_COMPONENT) */

    return (bytesRead);    
}


/** ----------------------------------------------------------------------------
 *  @name   DBL_Write
 *
 *  @desc   This routine is used to perform a write operation. This is 
 *          dummy for our implementation.
 *
 *  @arg    DWORD hOpenContext
 *              Device Driver Context Object
 *          LPVOID pBuffer
 *              Pointer to the Buffer
 *          DWORD Count
 *              Count of bytes
 *
 *  @ret    DWORD
 *              Operation successfully completed.
 *          DSP_EFAIL
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
DWORD
DBL_Write (DWORD hOpenContext, LPCVOID pBuffer, DWORD Count )
{
   
    
    return (0);     
}

/** ----------------------------------------------------------------------------
 *  @name   DBL_Seek
 *
 *  @desc   This routine is used to perform a seek operation. 
 *          This is dummy for our implementation.
 *
 *  @arg    DWORD hOpenContext
 *              Device Driver Context Object
 *          long Amount
 *              Amount by which to perform Seek
 *          WORD Type
 *              Seek Type
 *
 *  @ret    DWORD
 *              Operation successfully completed.
 *          0
 *              Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
DWORD
DBL_Seek (DWORD hOpenContext, long Amount, WORD Type )
{
    DWORD filePos = 0;
    
    
    
    return (filePos);     
}

/** ----------------------------------------------------------------------------
 *  @name   DBL_PowerUp
 *
 *  @desc   This routine is used to power up the device. This is dummy for 
 *          our implementation.
 *
 *  @arg    DWORD hDeviceContext
 *              Device Driver Context Object
 *
 *  @ret    None
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
void
DBL_PowerUp (DWORD hDeviceContext)
{
    
    TRC_0ENTER("LINK: +DBL_PowerUp\r\n");
    
    TRC_0LEAVE("LINK: -DBL_PowerUp\r\n");
    
    return ;     
}


/** ----------------------------------------------------------------------------
 *  @name   DBL_PowerDown
 *
 *  @desc   This routine is used to power down the device. This is dummy 
 *          for our implementation.
 *
 *  @arg    DWORD hDeviceContext
 *              Device Driver Context Object
 *
 *  @ret    None
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
void
DBL_PowerDown (DWORD hDeviceContext)
{
    
}
/** ----------------------------------------------------------------------------
 *  @name   DBL_IOControl
 *
 *  @desc   This routine is used to send a command to the PMGR Driver. 
 *          This function will be internally invoked when the DSP BIOS LINK 
 *          API component performs a DeviceIoControl() call in the 
 *          DRV_API call. 
 *
 *  @arg    DWORD hDeviceContext
 *              Pointer to the Device Driver Context Object.
 *          DWORD dwCode
 *              I/O control operation to perform
 *          PBYTE pBufIn
 *              Pointer to the buffer containing data to transfer to the 
 *              device
 *          DWORD dwLenIn
 *              Number of bytes of data in the buffer specified for pBufIn.
 *          PBYTE pBufOut
 *              Pointer to the buffer used to transfer the output data 
 *              from the device.
 *          DWORD dwLenOut
 *              Maximum number of bytes in the buffer specified by pBufOut.
 *          PDWORD pdwActualOut
 *              Pointer to the DWORD buffer that this function uses to 
 *              return the actual number of bytes received from the device.
 *
 *  @ret    BOOL
 *              TRUE  Operation successfully completed.
 *              FALSE Operation failed.
 *
 *  @enter  None.
 *
 *  @leave  None.
 *
 *  @see    None.
 *  ----------------------------------------------------------------------------
 */
EXPORT_API
BOOL
DBL_IOControl (DWORD hOpenContext, DWORD dwCode, PBYTE pBufIn, DWORD dwLenIn,
               PBYTE pBufOut, DWORD dwLenOut, PDWORD pdwActualOut, LPOVERLAPPED lpOverlapped )
{
                   
    DSP_STATUS status   = DSP_SOK ;
    BOOL       osStatus = FALSE   ;

    TRC_4ENTER ("DBL_IOControl", dwCode, pBufIn, dwLenIn, dwLenOut) ;


    if ( (pBufIn == NULL) || 
         (dwLenIn < sizeof (CMD_Args) ) )
    {
    }
    else {
        status = DRV_CallAPI (dwCode, (CMD_Args *) pBufIn) ;
    
        if (DSP_SUCCEEDED (status)) {
            osStatus = TRUE ;
        }
    }


    TRC_1LEAVE ("DBL_IOControl", status) ;

    return (osStatus) ;                   
                   
}

#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
