/** ============================================================================
 *  @file   main.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/message_multiapp/WinCE/
 *
 *  @desc   WinCE specific implementation. This is used to call multiple instances of
 *          message_multi app and a single instance of message_multidrv kernel driver.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <windows.h>



#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

#define MAX_APPS  16
#define MAXSTRLENGTH        256

typedef unsigned char       Uint8 ;     /*  8 bit value */
typedef struct inargs_s
{
    unsigned char dspExecutable[100];
    unsigned char strNumIterations[10];
    unsigned char strAppNum[4];
    unsigned char strProcessorId[4];
}inargs_t;


#define MESSAGE_APP_START 1
#define MESSAGE_APP_STOP  2

void start_driver(inargs_t * pInArgs)
{
    HANDLE driverHandle;
    DWORD  osStatus;    
    driverHandle = CreateFile( _T("MMG1:"), 
                                GENERIC_READ | GENERIC_WRITE, 0, NULL,
                                OPEN_EXISTING, 0, 0 );
    
    osStatus = DeviceIoControl ( driverHandle,
                                  MESSAGE_APP_START,
                                  (LPVOID)pInArgs,sizeof(inargs_t), NULL, 0, NULL, NULL);
    if (osStatus == FALSE) 
    {
        RETAILMSG(TRUE, (L"Couldnt send IOControl to MSG DLL\n"));    
    }
    return;
    
}


/** ============================================================================
 *  @func   main
 *
 *  @desc   Entry point for the application
 *
 *  @modif  None
 *  ============================================================================
 */
int main (int argc, char ** argv)
{
    char * dspExecutable    = NULL ;
    char * strNumIterations = NULL ;
    char * strAppNum        = NULL ;
    Uint8   appNum           = 0 ;
    char * strProcessorId   = NULL ;
    Uint8 processorId        = 0    ;
    PROCESS_INFORMATION procInfo[MAX_APPS];
    //char  buffer[MAXSTRLENGTH];
	WCHAR  buffer[MAXSTRLENGTH];
    int i;
    BOOL  status[MAX_APPS];
    DWORD retCode;
	HANDLE handles[MAX_APPS];
    inargs_t inArgs;
                

    if ((argc != 5) && (argc != 4)) {
        RETAILMSG(TRUE, (L"Usage : %hs <absolute path of DSP executable> "
               L"<number of transfers> <Number of Processes 1 -> %d>"
               L"<DSP Processor Id>\n"                
               L"For infinite transfers, use value of 0 for <number of transfers>\n",
               L"if x processes specified, then (x-1) are user process and x is kernel driver>\n",
               argv [0],
               MAX_APPS )) ;                
        RETAILMSG(TRUE, (L"For DSP Processor Id,"
           L"\n\t use value of 0  if sample needs to be run on DSP 0 "
           L"\n\t use value of 1  if sample needs to be run on DSP 1"
           L"\n\t For single DSP configuration this is optional argument\n"));        
    }
    else {
        dspExecutable    = argv [1] ;
        strNumIterations = argv [2] ;
        strAppNum        = argv [3] ;
        appNum           = atoi(argv [3]) ;

        if( argc == 4) {
            strProcessorId   = "0" ;
            processorId      = 0   ;
        }
        else if ( argc == 5 ) {
            strProcessorId = argv [4] ;
            processorId    = atoi (argv [4]) ;
        }
        strcpy(inArgs.dspExecutable,dspExecutable);
        strcpy(inArgs.strNumIterations,strNumIterations);
        strcpy(inArgs.strAppNum,strAppNum);
        strcpy(inArgs.strProcessorId,strProcessorId);        
        start_driver(&inArgs);
        
        if (processorId < MAX_PROCESSORS) {
            for (i=0; i<appNum-1; i++){   
                wsprintf(buffer, L"%S %S %d",dspExecutable,strNumIterations,(i+1));  
				//mbstowcs(wbuffer,buffer,MAXSTRLENGTH);
                status[i]=CreateProcess(L"messagemultigpp.exe",
                              buffer,
                              NULL,
                              NULL,
                              FALSE,
                              0,
                              NULL,
                              NULL,
                              NULL,
                              &procInfo[i]
                            );
                if (status[i]==0)
                {
                    RETAILMSG(TRUE, (L"Couldnt create process %d\n",i));                    
                }
				handles[i]=procInfo[i].hProcess;
            }
           
			for (i=0; i<appNum-1; i++){   
				retCode=WaitForMultipleObjects(appNum-1,handles,FALSE,INFINITE); 
				if (retCode == WAIT_FAILED)
				{
				   RETAILMSG(TRUE, (L"Wait failed for process %d with retCode %d and errCode %d\n",i,retCode,GetLastError()));   
				}
			}
        
        }

    }

    return 0 ;
}



#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
