/** ============================================================================
 *  @file   main.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/message_multidrv/WinCE/
 *
 *  @desc   Wince specific implementation of message sample application's driver.
 *
 *  @ver    1.65.00.03
 *  ============================================================================
 *  Copyright (C) 2002-2009, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <gpptypes.h>
#include <dsplink.h>
#include <errbase.h>

/*  ----------------------------------- Application Header            */
#include <message_os.h>
#include <message.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */



/** ============================================================================
 *  @func  DllEntry
 *
 *  @desc  Device driver DLL entry point. Any specific intialization is done
 *         here. The debug zones and GT are initialized here. The Successful
 *         loading of the DLL depends upon the retun value of any initialization.
 * 
 *  ============================================================================
 */ 
BOOL WINAPI DllEntry(HINSTANCE hDllInstance, INT nReason,
                         LPVOID lpvReserved)
{
    BOOL bReturnStatus = TRUE;

    MESSAGEMULTI_0Print("MESSAGE: +DllEntry\n");

    switch (nReason) {
        case DLL_PROCESS_ATTACH:
            DisableThreadLibraryCalls((HMODULE)hDllInstance);
            MESSAGEMULTI_0Print("MESSAGE: DllEntry DLL_PROCESS_ATTACH\n");
            break;

        case DLL_PROCESS_DETACH:
            MESSAGEMULTI_0Print("MESSAGE: DllEntry DLL_PROCESS_DETACH\n");
            break;

        case DLL_THREAD_ATTACH:
            MESSAGEMULTI_0Print("MESSAGE: DllEntry DLL_THREAD_ATTACH\n");
            break;

        case DLL_THREAD_DETACH:
            MESSAGEMULTI_0Print("MESSAGE: DllEntry DLL_THREAD_DETACH\n");
            break;

    }
    MESSAGEMULTI_0Print("MESSAGE: -DllEntry\n");

    return (bReturnStatus);
}


EXPORT_API
DWORD
MMG_Init(LPCTSTR pContext, LPCVOID lpvBusContext) 
{
    return 0x1;
}

EXPORT_API
BOOL 
MMG_Deinit (DWORD hDeviceContext) 
{
    return TRUE;
}

EXPORT_API
DWORD
MMG_Open (DWORD hDeviceContext, DWORD AccessCode, DWORD ShareMode ) 
{
    return 0x2;
}

EXPORT_API
BOOL
MMG_Close (DWORD hOpenContext)
{
    return TRUE;
}

EXPORT_API
DWORD
MMG_Read (DWORD hOpenContext, LPVOID pBuffer, DWORD Count )
{
    return 0;
}

EXPORT_API
DWORD
MMG_Write (DWORD hOpenContext, LPCVOID pBuffer, DWORD Count )
{ 
    return (0);     
}

EXPORT_API
DWORD
MMG_Seek (DWORD hOpenContext, long Amount, WORD Type )
{
    return (0);     
}

EXPORT_API
void
MMG_PowerUp (DWORD hDeviceContext)
{    
    return ;     
}

EXPORT_API
void
MMG_PowerDown (DWORD hDeviceContext)
{
    return;
}


typedef struct inargs_s
{
    Char8 dspExecutable[100];
    Char8 strNumIterations[10];
    Char8 strAppNum[4];
    Char8 strProcessorId[4];
}inargs_t;

inargs_t inArgs;

DWORD Start_Message_Main(LPVOID arg)
{
    inargs_t * pInArgs = (inargs_t *)arg;
    MESSAGEMULTI_Main (pInArgs->dspExecutable,pInArgs->strNumIterations,pInArgs->strAppNum,pInArgs->strProcessorId);
    return 0;
}

#define MESSAGE_MULTI_APP_START 1
#define MESSAGE_MULTI_APP_STOP  2


EXPORT_API
BOOL
MMG_IOControl (DWORD hOpenContext, DWORD dwCode, PBYTE pBufIn, DWORD dwLenIn,
               PBYTE pBufOut, DWORD dwLenOut, PDWORD pdwActualOut )
{
    HANDLE threadHandle;
    BOOL  fRet = FALSE;

    
    MESSAGEMULTI_1Print("MESSAGE: IOControl: %d\n",dwCode);    

    switch (dwCode) 
	{
	    case MESSAGE_MULTI_APP_START:
        {
            inargs_t *localInArgs=(inargs_t *)pBufIn;
            strcpy(inArgs.dspExecutable,localInArgs->dspExecutable);
            strcpy(inArgs.strNumIterations,localInArgs->strNumIterations);
            strcpy(inArgs.strAppNum,localInArgs->strAppNum);
            strcpy(inArgs.strProcessorId,localInArgs->strProcessorId);  
            threadHandle=CreateThread(NULL,0,
                        Start_Message_Main,
                        (LPVOID)&inArgs, 
                        CREATE_SUSPENDED,
                        NULL) ;
            ResumeThread(threadHandle);
            fRet=TRUE;
        }
        break;
        
        case MESSAGE_MULTI_APP_STOP:
            fRet=TRUE;
        break;
    }
    
    MESSAGEMULTI_1Print("MESSAGE: IOControl done: %d\n",dwCode);    
    return fRet;
}





#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
