#   ============================================================================
#   @file   jacinto1.mk
#
#   @path   $(DSPLINK)/make/PrOS/
#
#   @desc   This makefile defines OS specific macros used by MAKE system for
#           the PrOS distribution for the Davinci platform.
#
#   @ver    1.65.00.03
#   ============================================================================
#   Copyright (C) 2002-2009, Texas Instruments Incorporated -
#   http://www.ti.com/
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#   
#   *  Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#   
#   *  Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#   
#   *  Neither the name of Texas Instruments Incorporated nor the names of
#      its contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#   
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#   ============================================================================


ifndef JACINTO_MK

define JACINTO_MK
endef


#   ============================================================================
#   Let the make system know that a specific distribution for the GPP OS
#   is being used.
#   ============================================================================
USE_DISTRIBUTION := 1


#   ============================================================================
#   Set the values of necessary variables to be used for the OS.
#   ============================================================================

#   ----------------------------------------------------------------------------
#   Base directory for the GPP OS
#   ----------------------------------------------------------------------------
BASE_BUILDOS    := L:\eSOL\eBinder\package

#   ----------------------------------------------------------------------------
#   Base for code generation tools - compiler, linker, archiver etc.
#   ----------------------------------------------------------------------------
BASE_CGTOOLS    := L:\eSOL\eBinder\cl\ARM\RVCT\Programs\2.2\503\win_32-pentium

#   ----------------------------------------------------------------------------
#   For PrFile support, the header file <pf_api.h> has a dependancy on generated
#   file eb_pf_config.h. This generated file is not part of Base OS but project
#   specific.
#   For using PrFile, please set this part to a eBinder project.
#   ----------------------------------------------------------------------------
EBINDER_PROJECTHOME := L:\eSOL\eBinder\projects\dsplink

#   ----------------------------------------------------------------------------
#   Base for Log library and header files.
#   ----------------------------------------------------------------------------
BASE_LOG    := c:\ti-tools\log


#   ----------------------------------------------------------------------------
#   Base directory for include files provided by GPP OS
#   ----------------------------------------------------------------------------

BASE_OSINC      := $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\include                              \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\include\bios                         \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\include\prkernelv4                   \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\include\prkernelv4\arm               \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\include\prkernelv4\include           \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\config                               \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\config\prkernelv4                    \
                   $(BASE_BUILDOS)\PP\KERNEL\ESOL_PRKERNELV4_ARM_RVCT\source\parts\config\prkernelv4\arm                \
                   $(EBINDER_PROJECTHOME)\build\parts\config                                                            \
                   $(BASE_BUILDOS)\PP\MW\ESOL_PRFILE2\source\parts\config                                               \
                   $(BASE_BUILDOS)\PP\MW\ESOL_PRFILE2\source\parts\include                                              \
                   $(BASE_BUILDOS)\PP\MW\ESOL_PRFILE2\source\parts\include\prfile2                                      \
                   $(BASE_BUILDOS)\BSP\TI_JACINTO_EVM_ARM_RVCT\source\parts\bios_ep\board\jacintoevm\include            \
                   $(BASE_BUILDOS)\BSP\TI_JACINTO_EVM_ARM_RVCT\source\parts\bios_ep\board\jacintoevm\include\prkernelv4 \
                   $(BASE_LOG)\include                                                                                  \
                   

OSINC_GENERIC   := $(BASE_OSINC)
OSINC_PLATFORM  := $(BASE_OSINC) $(BASE_CGTOOLS)\include

ifneq ("$(VARIANT)", "")
OSINC_VARIANT   := $(BASE_OSINC)
endif


#   ----------------------------------------------------------------------------
#   Base directory for libraries provided by GPP OS
#   ----------------------------------------------------------------------------

BASE_OSLIB      := $(BASE_BUILDOS)\lib                          \
                   $(BASE_LOG)\include


OSLIB_GENERIC   := $(BASE_OSLIB)
OSLIB_PLATFORM  := $(BASE_OSLIB)

ifneq ("$(VARIANT)", "")
OSLIB_VARIANT   := $(BASE_OSLIB)
endif


#   ============================================================================
#   COMPILER
#   ============================================================================

#   ----------------------------------------------------------------------------
#   Name of the compiler
#   ----------------------------------------------------------------------------
COMPILER        := $(BASE_CGTOOLS)\armcc

#   ----------------------------------------------------------------------------
#   Command line switches used by the compiler
#
#   CC_SW_DEF       Command line defines
#   CC_SW_INC       Search path for header files
#   CC_SW_OBJ       Create object file
#   CC_SW_DEB       Include debug information
#   ----------------------------------------------------------------------------
CC_SW_DEF       := -D
CC_SW_INC       := -I
CC_SW_OBJ       := -o
CC_SW_DEB       := -g

#   ----------------------------------------------------------------------------
#   Standard flags for the compiler
#   ----------------------------------------------------------------------------
STD_CC_FLAGS    := -c --cpu 5TE --apcs /interwork --c90 -O2

#   ----------------------------------------------------------------------------
#   Standard definitions for the compiler
#   ----------------------------------------------------------------------------
STD_CC_DEFNS    := -DOS_PROS -DLOG_EVENT -D__APCS_INTERWORK -D__TARGET_CPU_generic -D__TARGET_ARCH_5TE -D__TARGET_FEATURE_DSPMUL -D__TARGET_FEATURE_DOUBLEWORD


#   ============================================================================
#   ARCHIVER
#   ============================================================================
ARCHIVER        := $(BASE_CGTOOLS)\armar

#   ----------------------------------------------------------------------------
#   Standard flags for the archiver
#   ----------------------------------------------------------------------------
STD_AR_FLAGS    := -r --create


#   ============================================================================
#   LINKER
#   ============================================================================
LINKER      := $(BASE_CGTOOLS)\armlink

#   ----------------------------------------------------------------------------
#   Command line switches used by the linker
#
#   LD_SW_LIB       Search path for libraries
#   LD_SW_OUT       Output filename
#   LD_SW_RELOC     Generate relocateable output
#   ----------------------------------------------------------------------------
LD_SW_LIB       :=
LD_SW_OUT       :=
LD_SW_RELOC     :=

#   ----------------------------------------------------------------------------
#   Standard flags for the linker
#   ----------------------------------------------------------------------------
STD_LD_FLAGS    :=


endif   # ifndef JACINTO_MK
