#   ============================================================================
#   @file   omap3530_wince6.0.mk
#
#   @path   $(DSPLINK)/make/WinCE/
#
#   @desc   This makefile defines OS specific macros used by MAKE system for
#           the OMAP 3530 Wince 6.0 distribution.
#
#   @ver    1.65.00.03
#   ============================================================================
#   Copyright (C) 2002-2009, Texas Instruments Incorporated -
#   http://www.ti.com/
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#   
#   *  Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#   
#   *  Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#   
#   *  Neither the name of Texas Instruments Incorporated nor the names of
#      its contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#   
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#   ============================================================================


ifndef OMAP3530_WINCE6_0_MK

define OMAP3530_WINCE6_0_MK
endef

#   ============================================================================
#   Let the make system know that a specific distribution for the GPP OS
#   is being used.
#   ============================================================================
USE_DISTRIBUTION := 1


#   ============================================================================
#   Set the values of necessary variables to be used for the OS.
#   ============================================================================

#   ----------------------------------------------------------------------------
#   Base directory for the GPP OS
#   ----------------------------------------------------------------------------
BASE_BUILDOS    := $(_WINCEROOT)

#   ----------------------------------------------------------------------------
#   Base for toolchain
#   ----------------------------------------------------------------------------
BASE_TOOLCHAIN  := $(_WINCEROOT)\SDK\BIN

#   ----------------------------------------------------------------------------
#   Base for code generation tools - compiler, linker, archiver etc.
#   ----------------------------------------------------------------------------
BASE_CGTOOLS    := $(_WINCEROOT)\SDK\BIN

#   ----------------------------------------------------------------------------
#   Base directory for include files provided by GPP OS
#   ----------------------------------------------------------------------------
BASE_OSINC       := $(_PROJECTROOT)\cesysgen\sdk\inc
BASE_OAKINC      := $(_PROJECTROOT)\cesysgen\oak\inc
BASE_DDKINC      := $(_PROJECTROOT)\cesysgen\ddk\inc

OSINC_GENERIC   := $(BASE_OSINC)
OSINC_PLATFORM  := $(_TARGETPLATROOT)\SRC\INC
OSINC_TARGET    := $(BASE_OAKINC)

ifneq ("$(VARIANT)", "")
OSINC_VARIANT   := $(BASE_OSINC)
endif


#   ----------------------------------------------------------------------------
#   Base directory for libraries provided by GPP OS
#   ----------------------------------------------------------------------------
BASE_OSLIB      := $(_TARGETPLATROOT)\LIB\$(_TGTCPU)\$(WINCEDEBUG)

OSLIB_GENERIC   := $(BASE_OSLIB)
OSLIB_PLATFORM  := $(BASE_OSLIB)


ifneq ("$(VARIANT)", "")
OSLIB_VARIANT   := $(BASE_OSLIB)
endif


#   ============================================================================
#   COMPILER
#   ============================================================================

#   ----------------------------------------------------------------------------
#   Name of the compiler
#   ----------------------------------------------------------------------------
COMPILER        := $(BASE_CGTOOLS)\I386\ARM\cl.exe
LD              := $(BASE_CGTOOLS)\I386\ARM\link.exe

CROSS_COMPILE   =


#   ----------------------------------------------------------------------------
#   Command line switches used by the compiler
#
#   CC_SW_DEF       Command line defines
#   CC_SW_INC       Search path for header files
#   CC_SW_OBJ       Create object file
#   CC_SW_DEB       Include debug information
#   ----------------------------------------------------------------------------
CC_SW_DEF       := -D
CC_SW_INC       := -I
CC_SW_OBJ       := -Fo
CC_SW_DEB       := -Zi



#   ----------------------------------------------------------------------------
#   Standard flags for the compiler
#   ----------------------------------------------------------------------------
STD_KRNL_FLAGS  := -nologo -WX -Zc:wchar_t- -Zc:forScope- -MT -GF -GR- -D_ARM_ -DARM -DARMV4I -DUNDER_CE=600 -D_WIN32_WCE=0x600 -DSTRICT -DWIN32 -QRarch4T -DTHUMBSUPPORT -QRinterwork-return -QRimplicit-import- -DUNICODE -D_UNICODE -DWINCEMACRO /Gy   /Uinterface -GS

STD_USER_FLAGS  := -nologo -WX -Zc:wchar_t- -Zc:forScope- -MT -GF -GR- -D_ARM_ -DARM -DARMV4I -DUNDER_CE=600 -D_WIN32_WCE=0x600 -DSTRICT -DWIN32 -QRarch4T -DTHUMBSUPPORT -QRinterwork-return -QRimplicit-import- -DUNICODE -D_UNICODE -DL0409 -DINTLSG_CODEPAGE=1252  -DWINCEOEM -DWINCEINTERNAL /Uinterface -GS


#   ----------------------------------------------------------------------------
#   Standard flags for the compiler when building an executable
#   ----------------------------------------------------------------------------
EXE_CC_FLAGS    :=

#   ----------------------------------------------------------------------------
#   Flags for the compiler when building a driver
#   ----------------------------------------------------------------------------
DRV_CC_FLAGS    :=

#   ----------------------------------------------------------------------------
#   Flags for the compiler when building a library
#   ----------------------------------------------------------------------------
LIB_CC_FLAGS    :=

#   ----------------------------------------------------------------------------
#   Standard definitions for the compiler
#   ----------------------------------------------------------------------------
STD_CC_DEFNS    := $(CC_SW_DEF)_CRT_SECURE_NO_DEPRECATE $(CC_SW_DEF)_USE_32BIT_TIME_T

#   ============================================================================
#   ARCHIVER2 - This denotes the archiver.
#   ============================================================================
ARCHIVER        := $(BASE_CGTOOLS)\I386\ARM\link.exe

#   ----------------------------------------------------------------------------
#   Standard flags for the archiver
#   ----------------------------------------------------------------------------
STD_AR_FLAGS    := -lib /MACHINE:ARM /SUBSYSTEM:WINDOWSCE,6.00 -nodefaultlib -ignore:4001 /nologo


#   ============================================================================
#   LINKER - The compiler is used for linking purpose as well.
#   ============================================================================
LINKER      := $(BASE_CGTOOLS)\I386\ARM\link.exe

#   ----------------------------------------------------------------------------
#   Command line switches used by the linker
#
#   LD_SW_LIB       Search path for libraries
#   LD_SW_OUT       Output filename
#   LD_SW_RELOC     Generate relocateable output
#   ----------------------------------------------------------------------------
LD_SW_LIB       := -L
LD_SW_OUT       := -out:
LD_SW_RELOC     :=

#   ----------------------------------------------------------------------------
#   Standard flags for the linker
#   ----------------------------------------------------------------------------
STD_LD_FLAGS    := /nologo /LARGEADDRESSAWARE /opt:ref -nodefaultlib -debug -debugtype:cv -incremental:no

#   ----------------------------------------------------------------------------
#   Flags for the linker when building an executable
#   ----------------------------------------------------------------------------
EXE_LD_FLAGS    := /base:0x00010000 -MERGE:.rdata=.text -align:4096 -ignore:4001,4070,4078,4086,4089,4096,4099,4108,4229 /STACK:65536,4096 /SUBSYSTEM:CONSOLE


#   ----------------------------------------------------------------------------
#   Flags for the linker when building a driver
#   ----------------------------------------------------------------------------
DRV_LD_FLAGS    := -entry:DllEntry /base:0x10000000 /STACK:65536,4096 -align:4096 -ignore:4001,4070,4078,4086,4089,4096,4099,4108,4229 /SUBSYSTEM:WINDOWSCE,6.00


#   ============================================================================
#   Post processing utilities for WinCE
#   ============================================================================
CMD_MODPOST     :=


endif   # ifndef OMAP3530_WINCE6_0_MK
